#!/usr/bin/python
# Copyright (c) 2011 Collabora Ltd. <http://www.collabora.co.uk/>
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from configfile import ConfigFile
from gi.repository import Gtk


class ConfigWizard():

    """Simple configuration wizard window."""

    def __init__(self, config_file_path):
        self._config_items = []
        self._config_entries = {}
        self._config_file_path = config_file_path
        self._config_file_obj = None

    """
    [ {item_label, item_type, item_name, item_with_value} , ... ]
    """

    def set_config_items(self, items):
        self._config_items = items
        keys = {}
        for i in self._config_items:
            keys[i["item_name"]] = {"type": i["item_type"]}
        self._valid_keys = keys

    def set_config_file_obj(self, obj):
        self._config_file_obj = obj

    def get_config_file_obj(self, obj):
        return self._config_file_obj

    def show(self, read_from_disc=False):

        if read_from_disc:
            self._config_file_obj = ConfigFile(self._config_file_path)
            self._config_file_obj.set_valid_keys(self._valid_keys)
            self._config_file_obj.load()
        else:
            if self._config_file_obj is None:
                raise RuntimeError("I need the run time obj")

        self._config_popup = Gtk.Window()
        self._config_popup.set_default_size(200, 200)
        self._config_popup.connect('delete_event', self._close_config_cb)
        table = Gtk.Table(12, 1, True)
        self._config_popup.add(table)

        row = 1
        for i in self._config_items:
            hbox = self._create_param(i)
            table.attach(hbox, 0, 1, row, row + 1, xpadding=5, ypadding=2)
            row = row + 1

        hbox = Gtk.HBox()
        save_button = Gtk.Button.new_with_label('Save')
        save_button.set_size_request(50, 15)
        save_button.connect('pressed', self._save_config_cb)
        hbox.add(save_button)
        cancel_button = Gtk.Button.new_with_label('Cancel')
        cancel_button.set_size_request(50, 15)
        cancel_button.connect('pressed', self._close_config_cb)
        hbox.add(cancel_button)
        table.attach(hbox, 0, 1, row, row + 1, xpadding=5, ypadding=2)

        self._config_popup.show_all()

    def _save_config_cb(self, widget):
        try:
            self._do_save_config()
        except Exception as e:
            w = Gtk.Window()
            l = Gtk.Label(e.message)
            w.add(l)
            w.show_all()
        finally:
            self._config_popup.hide()

    def _do_save_config(self):
        for i in self._config_items:
            param_name = i["item_name"]
            v = self._config_entries[param_name]
            if v.__class__ is Gtk.Entry:
                value = v.get_text()
            elif v.__class__ is Gtk.CheckButton:
                value = v.get_active()
            else:
                raise RuntimeError("Don't recognize the class %s" % type(v))
            self._config_file_obj.set(param_name, value)

        self._config_file_obj.save()

    """
      {item_label, item_type, item_name, item_with_value}
    """

    def _create_param(self, opts):
        param_name = opts["item_name"]
        with_value = opts["item_with_value"] if "item_with_value" in opts \
            else True
        hbox = Gtk.HBox()
        if opts["item_type"] == "text":
            entry = Gtk.Entry()
            entry.set_size_request(150, 25)
            if with_value:
                value = self._config_file_obj.get(param_name, True)
                entry.set_text(str(value))
        elif opts["item_type"] == "boolean":
            entry = Gtk.CheckButton()
            if with_value:
                value = self._config_file_obj.get(param_name, True)
                entry.set_active(value)
        self._config_entries[param_name] = entry
        label = Gtk.Label(opts["item_label"] + ': ')
        label.set_alignment(1.0, 0.5)
        label.set_size_request(100, 25)
        hbox.add(label)
        hbox.add(entry)
        return hbox

    def _close_config_cb(self, widget, event=None):
        self._config_popup.hide()


def test_wizard_from_config_file_obj(test_config_file):
    keys = {}
    keys["nick"] = {"type": "text"}
    keys["account_id"] = {"type": "text"}
    keys["server"] = {"type": "text"}
    keys["port"] = {"type": "text"}
    keys["password"] = {"type": "text"}
    keys["register"] = {"type": "text"}

    c = ConfigFile(test_config_file)
    c.set_valid_keys(keys)
    c.set("nick", "rgs")
    c.set("account_id", "rgs@andromeda")
    c.set("server", "andromeda")
    c.set("port", 5223)
    c.set("password", "97c74fa0dc3b39b8c87f119fa53cced2b7040786")
    c.set("register", True)

    c.save()

    c = ConfigFile(test_config_file)
    c.set_valid_keys(keys)
    c.load()

    config_w = ConfigWizard(test_config_file)
    config_items = [
        {"item_label": "Nickname", "item_type": "text", "item_name": "nick"},
        {"item_label": "Account ID", "item_type": "text",
         "item_name": "account_id"},
        {"item_label": "Server", "item_type": "text", "item_name": "server"},
        {"item_label": "Port", "item_type": "text", "item_name": "port"},
        {"item_label": "Password", "item_type": "text",
         "item_name": "password"},
        {"item_label": "Register", "item_type": "text",
         "item_name": "register"}]
    config_w.set_config_items(config_items)
    config_w.set_config_file_obj(c)
    config_w.show()


def test_wizard_from_config_file_path(test_config_file):
    keys = {}
    keys["nick"] = {"type": "text"}
    keys["account_id"] = {"type": "text"}
    keys["server"] = {"type": "text"}
    keys["port"] = {"type": "text"}
    keys["password"] = {"type": "text"}
    keys["register"] = {"type": "text"}

    c = ConfigFile(test_config_file)
    c.set_valid_keys(keys)
    c.set("nick", "rgs")
    c.set("account_id", "rgs@andromeda")
    c.set("server", "andromeda")
    c.set("port", 5223)
    c.set("password", "97c74fa0dc3b39b8c87f119fa53cced2b7040786")
    c.set("register", True)

    c.save()

    config_w = ConfigWizard(test_config_file)
    config_items = [
        {"item_label": "Nickname", "item_type": "text", "item_name": "nick"},
        {"item_label": "Account ID", "item_type": "text",
         "item_name": "account_id"},
        {"item_label": "Server", "item_type": "text", "item_name": "server"},
        {"item_label": "Port", "item_type": "text", "item_name": "port"},
        {"item_label": "Password", "item_type": "text",
         "item_name": "password"},
        {"item_label": "Register", "item_type": "text",
         "item_name": "register"}]
    config_w.set_config_items(config_items)
    config_w.show(True)


if __name__ == "__main__":
    # test_wizard_from_config_file_obj("/tmp/configwizard.test.0001")
    test_wizard_from_config_file_path("/tmp/configwizard.test.0002")
    Gtk.main()



# Copyright (c) 2013 Marion Zepf

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

""" Python export tool """

import ast
from gettext import gettext as _
from os import linesep
from os import path, pardir
import re
import traceback
from .util import codegen

# from ast_pprint import * # only used for debugging, safe to comment out

from .talogo import LogoCode
from .taprimitive import (ast_yield_true, Primitive, PyExportError,
                         value_to_ast)
from .tautils import (find_group, find_top_block, get_stack_name)
from .tawindow import plugins_in_use


_SETUP_CODE_START = """\
#!/usr/bin/env python
# -*- coding: utf-8 -*-

_INSTALL_PATH = '/usr/share/sugar/activities/TurtleArt.activity'
_ALTERNATIVE_INSTALL_PATH = \\
    '/usr/local/share/sugar/activities/TurtleArt.activity'

import os, sys, dbus
paths = []

paths.append('../%s.activity')
paths.append(os.path.expanduser('~') + '/Activities/%s.activity')
paths.append('/usr/share/sugar/activities/%s.activity')
paths.append('/usr/local/share/sugar/activities/%s.activity')
paths.append(
    '/home/broot/sugar-build/build/install/share/sugar/activities/%s.activity')
""" + \
    "paths.append('%s')" % \
    path.abspath(path.join(path.dirname(__file__), pardir)) + \
    """\

flag = False
for path in paths:
    for activity in ['TurtleBots', 'TurtleBlocks']:
        p = (path % activity) if "%" in path else path

        if os.path.exists(p) and p not in sys.path:
            flag = True
            sys.path.insert(0, p)

if not flag:
    print 'This code require the Turtle Blocks/Bots activity to be installed.'
    exit(1)

from time import *
from random import uniform
from math import *

from pyexported.window_setup import *


tw = get_tw()

BOX = {}
ACTION = {}

global_objects = None
turtles = None
canvas = None
logo = None
"""

_SETUP_CODE_END = """\

if __name__ == '__main__':
    tw.lc.start_time = time()
    tw.lc.icall(start)
    gobject.idle_add(tw.lc.doevalstep)
    gtk.main()
"""
_ACTION_STACK_START = """\
def %s():
"""
_START_STACK_START_ADD = """\
    tw.start_plugins()
    global global_objects,turtles,canvas,logo
    global_objects = tw.get_global_objects()
    turtles = tw.turtles
    canvas = tw.canvas
    logo = tw.lc
    logo.boxes = BOX
"""
_ACTION_STACK_PREAMBLE = """\
    turtle = turtles.get_active_turtle()
"""
_ACTION_STACK_END = """\
ACTION["%s"] = %s
"""
# character that is illegal in a Python identifier
PAT_IDENTIFIER_ILLEGAL_CHAR = re.compile("[^A-Za-z0-9_]")


def save_python(tw):
    """ Find all the action stacks and turn each into Python code """
    all_blocks = tw.just_blocks()
    blocks_name = []
    for block in all_blocks:
        blocks_name.append(block.name)

    if 'start' not in blocks_name:
        return None

    blocks_covered = set()
    tops_of_stacks = []
    for block in all_blocks:
        if block not in blocks_covered:
            top = find_top_block(block)
            tops_of_stacks.append(top)
            block_stack = find_group(top)
            blocks_covered.update(set(block_stack))

    snippets = [_SETUP_CODE_START]

    for k in plugins_in_use:
        snippets.append('%s = None\n' % (k.lower(),))

    snippets.append('\n')

    for block in tops_of_stacks:
        stack_name = get_stack_name(block)
        if stack_name:
            pythoncode = _action_stack_to_python(block, tw, name=stack_name)
            snippets.append(pythoncode)
            snippets.append(linesep)
    snippets.append(_SETUP_CODE_END)
    return ''.join(snippets)


def _action_stack_to_python(block, tw, name='start'):
    """ Turn a stack of blocks into Python code
    name -- the name of the action stack (defaults to "start") """

    if isinstance(name, int):
        name = float(name)
    if not isinstance(name, str):
        name = str(name)

    # traverse the block stack and get the AST for every block
    ast_list = _walk_action_stack(block, tw.lc)
    if not ast_list or not isinstance(ast_list[-1], ast.Yield):
        ast_list.append(ast_yield_true())
    action_stack_ast = ast.Module(body=ast_list)

    # serialize the ASTs into python code
    generated_code = codegen.to_source(action_stack_ast)

    # wrap the action stack setup code around everything
    name_id = _make_identifier(name)
    if name == 'start':
        pre_preamble = _START_STACK_START_ADD
        for k in plugins_in_use:
            pre_preamble += '    global %s\n' % (k.lower(),)
            pre_preamble += "    %s = global_objects['%s']\n" % (k.lower(), k)
    else:
        pre_preamble = ''
    generated_code = _indent(generated_code, 1)
    if generated_code.endswith(linesep):
        newline = ''
    else:
        newline = linesep
    snippets = [_ACTION_STACK_START % (name_id),
                pre_preamble,
                _ACTION_STACK_PREAMBLE,
                generated_code,
                newline,
                _ACTION_STACK_END % (name, name_id)]
    return ''.join(snippets)


def _walk_action_stack(top_block, lc, convert_me=True):
    """ Turn a stack of blocks into a list of ASTs
    convert_me -- convert values and Primitives to ASTs or return them
        unconverted? """
    block = top_block

    # value blocks don't have a primitive
    # (but constant blocks (colors, screen dimensions, etc.) do)
    if block.is_value_block():
        raw_value = block.get_value(add_type_prefix=False)
        if convert_me:
            value_ast = value_to_ast(raw_value)
            if value_ast is not None:
                return [value_ast]
            else:
                return []
        else:
            if raw_value is not None:
                return [raw_value]
            else:
                return []

    def _get_prim(block):
        prim = lc.get_prim_callable(block.primitive)
        # fail gracefully if primitive is not a Primitive object
        if not isinstance(prim, Primitive):
            raise PyExportError(_('block is not exportable'), block=block)
        return prim

    prim = _get_prim(block)

    ast_list = []
    arg_asts = []

    def _finish_off(block, prim=None):
        """ Convert block to an AST and add it to the ast_list. Raise a
        PyExportError on failure. """
        if prim is None:
            prim = _get_prim(block)
        if convert_me:
            if prim.export_me:
                try:
                    new_ast = prim.get_ast(*arg_asts)
                except ValueError:
                    traceback.print_exc()
                    raise PyExportError(_('error while exporting block'),
                                        block=block)
                if isinstance(new_ast, (list, tuple)):
                    ast_list.extend(new_ast)
                elif new_ast is not None:
                    ast_list.append(new_ast)
            elif arg_asts:  # TODO do we ever get here?
                new_ast = ast.List(elts=arg_asts, ctx=ast.Load)
                ast_list.append(new_ast)
        else:
            ast_list.append((prim, ) + tuple(arg_asts))

    # skip the very first dock/ connection - it's either the previous block or
    # the return value of this block
    dock_queue = block.docks[1:]
    conn_queue = block.connections[1:]
    while dock_queue and conn_queue:
        dock = dock_queue.pop(0)
        conn = conn_queue.pop(0)
        if conn is None or dock[0] == 'unavailable':
            continue
        elif not dock_queue and dock[0] == 'flow':
            # finish off this block
            _finish_off(block, prim)
            arg_asts = []
            # next block
            block = conn
            prim = _get_prim(block)
            dock_queue = block.docks[1:]
            conn_queue = block.connections[1:]
        else:
            # embedded stack of blocks (body of conditional or loop) or
            # argument block
            if dock[0] == 'flow':
                # body of conditional or loop
                new_arg_asts = _walk_action_stack(conn, lc,
                                                  convert_me=convert_me)
                if (prim == LogoCode.prim_loop and
                        not isinstance(new_arg_asts[-1], ast.Yield)):
                    new_arg_asts.append(ast_yield_true())
                arg_asts.append(new_arg_asts)
            else:
                # argument block
                new_arg_asts = _walk_action_stack(conn, lc, convert_me=False)
                arg_asts.append(*new_arg_asts)

    # finish off last block
    _finish_off(block, prim)

    return ast_list


def _make_identifier(name):
    """ Turn name into a Python identifier name by replacing illegal
    characters """
    replaced = re.sub(PAT_IDENTIFIER_ILLEGAL_CHAR, '_', name)
    # TODO find better strategy to avoid number at beginning
    if re.match('[0-9]', replaced):
        replaced = '_' + replaced
    return replaced


def _indent(code, num_levels=1):
    """ Indent each line of code with num_levels * 4 spaces
    code -- some python code as a (multi-line) string """
    indentation = " " * (4 * num_levels)
    line_list = code.split(linesep)
    new_line_list = []
    for line in line_list:
        new_line_list.append(indentation + line)
    return linesep.join(new_line_list)



# Copyright (c) 2009-11, Walter Bender

# This procedure is invoked when the user-definable block on the
# "extras" palette is selected and expanded to 3 arguments.

# Usage: Import this code into a Python (user-definable) block.
# In using SVG, it is sometimes useful to divide your drawing into
# groups of elements. You can do that by inserting <g> </g> around
# sections of your code.
#
# Place the svg_start_group.py block at the point in your program
# where you'd like to start a group in your SVG output.
#
# Be sure to use the corresponding svg_end_group.py block to close
# the SVG group definition.


def myblock(tw, x):
    ''' Add end group to SVG output '''

    tw.svg_string += '</g>'



# Copyright (c) 2009-11, Walter Bender, Tony Forster

# This procedure is invoked when the user-definable block on the
# "extras" palette is selected.

# Usage: Import this code into a Python (user-definable) block; when
# this code is run, the current mouse status will be pushed to the
# FILO heap. If a mouse button event occurs, a y, x, and 1 are pushed
# to the heap. If no button is pressed, 0 is pushed to the heap.

# To use these data, pop the heap in a compare block to determine if a
# button has been pushed. If a 1 was popped from the heap, pop the x
# and y coordinates.


def myblock(tw, x):  # ignore second argument
    ''' Push mouse event to stack '''

    if tw.mouse_flag == 1:
        # push y first so x will be popped first
        tw.lc.heap.append((tw.canvas.height / 2) - tw.mouse_y)
        tw.lc.heap.append(tw.mouse_x - (tw.canvas.width / 2))
        tw.lc.heap.append(1)  # mouse event
        tw.mouse_flag = 0
    else:
        tw.lc.heap.append(0)  # no mouse event



# Copyright (c) 2010-11, Walter Bender, Tony Forster
#
# This Python block writes serial output to a USB port and pushes
# serial input to the heap.
#
# To use this block:
#    (1) import this file into a Python Block;
#    (2) pass text strings as an argument
#    (3) use a Pop Block to retrieve any strings input from serial device.


def myblock(tw, args):  # x is the string to transmit
    import serial  # you may need to install this library

    # serial device on USB, 9600 baud
    ser = serial.Serial('/dev/ttyUSB0', 9600, timeout=1)

    ser.write(str(args[0]))  # send string x
    st = ser.read(1000)  # read up to 1000 bytes
    tw.lc.heap.append(st)  # append to heap
    ser.close()



#! /usr/bin/python
#
# Author:  Arjun Sarwal   arjun@laptop.org
# Copyright (C) 2007, Arjun Sarwal
# Copyright (C) 2009-11 Walter Bender
# Copyright (C) 2009, Benjamin Berg, Sebastian Berg
# Copyright (C) 2009, Sayamindu Dasgupta
# Copyright (C) 2010, Sascha Silbe
# Copyright (C) 2016, James Cameron [GStreamer 1.0]
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# You should have received a copy of the GNU General Public License
# along with this library; if not, write to the Free Software
# Foundation, 51 Franklin Street, Suite 500 Boston, MA 02110-1335 USA


from gi.repository import Gst
Gst.init(None)

from numpy import fromstring
import subprocess
import traceback
from string import find
from threading import Timer
from numpy import append
from TurtleArt.taconstants import XO1, XO4

import logging

log = logging.getLogger('turtleart-activity')
log.setLevel(logging.ERROR)

# Initial device settings
RATE = 48000
MIC_BOOST = True
DC_MODE_ENABLE = False
CAPTURE_GAIN = 50
BIAS = True

# Setting on quit
QUIT_MIC_BOOST = False
QUIT_DC_MODE_ENABLE = False
QUIT_CAPTURE_GAIN = 100
QUIT_BIAS = True

SENSOR_AC_NO_BIAS = 'external'
SENSOR_AC_BIAS = 'sound'
SENSOR_DC_NO_BIAS = 'voltage'
SENSOR_DC_BIAS = 'resistance'


class AudioGrab():
    """ The interface between measure and the audio device """

    def __init__(self, callable1, parent,
                 mode=None, bias=None, gain=None, boost=None):
        """ Initialize the class: callable1 is a data buffer;
            activity is the parent class """

        self.callable1 = callable1
        self.parent = parent
        self.rate = RATE
        if self.parent.hw in [XO1, XO4]:
            self.channels = 1
        else:
            self.channels = None
        if self.channels is None:
            self.channels = 2

        self.we_are_logging = False
        self._log_this_sample = False
        self._logging_timer = None
        self._logging_counter = 0
        self._image_counter = 0
        self._logging_interval = 0
        self._channels_logged = []
        self._busy = False

        self._dont_queue_the_buffer = False

        #self._display_counter = DISPLAY_DUTY_CYCLE

        # self.activity.wave.set_channels(self.channels)
        for i in range(self.channels):
            self._channels_logged.append(False)

        # Set mixer to known state
        self.set_dc_mode(DC_MODE_ENABLE)
        self.set_bias(BIAS)
        self.set_capture_gain(CAPTURE_GAIN)
        self.set_mic_boost(MIC_BOOST)

        self._set_sensor_type(mode, bias, gain, boost)
        self.master = self.get_master()
        self.dc_mode = self.get_dc_mode()
        self.bias = self.get_bias()
        self.capture_gain = self.get_capture_gain()
        self.mic_boost = self.get_mic_boost()

        # Set up gstreamer pipeline
        self._pad_count = 0
        self.pads = []
        self.queue = []
        self.fakesink = []
        self.pipeline = Gst.Pipeline.new('pipeline')
        self.alsasrc = Gst.ElementFactory.make('alsasrc', 'alsa-source')
        self.pipeline.add(self.alsasrc)
        self.caps1 = Gst.ElementFactory.make('capsfilter', 'caps1')
        self.pipeline.add(self.caps1)
        caps_str = 'audio/x-raw,rate=(int)%d,channels=(int)%d,depth=(int)16' % (
            RATE, self.channels)
        self.caps1.set_property('caps', Gst.caps_from_string(caps_str))
        if self.channels == 1:
            self.fakesink.append(Gst.ElementFactory.make('fakesink', 'fsink'))
            self.pipeline.add(self.fakesink[0])
            self.fakesink[0].connect('handoff', self.on_buffer, 0)
            self.fakesink[0].set_property('signal-handoffs', True)
            self.alsasrc.link(self.caps1)
            self.caps1.link(self.fakesink[0])
        else:
            if not hasattr(self, 'splitter'):
                self.splitter = Gst.ElementFactory.make('deinterleave')
                self.pipeline.add(self.splitter)
                self.splitter.set_properties('keep-positions=true', 'name=d')
                self.splitter.connect('pad-added', self._splitter_pad_added)
                self.alsasrc.link(self.caps1)
                self.caps1.link(self.splitter)
            for i in range(self.channels):
                self.queue.append(Gst.ElementFactory.make('queue'))
                self.pipeline.add(self.queue[i])
                self.fakesink.append(Gst.ElementFactory.make('fakesink'))
                self.pipeline.add(self.fakesink[i])
                self.fakesink[i].connect('handoff', self.on_buffer, i)
                self.fakesink[i].set_property('signal-handoffs', True)

    def _unlink_sink_queues(self):
        ''' Build the sink pipelines '''

        # If there were existing pipelines, unlink them
        for i in range(self._pad_count):
            log.debug('unlinking old elements')
            try:
                self.splitter.unlink(self.queue[i])
                self.queue[i].unlink(self.fakesink[i])
            except BaseException:
                traceback.print_exc()

        # Build the new pipelines
        self._pad_count = 0
        self.pads = []
        log.debug('building new pipelines')

    def _splitter_pad_added(self, element, pad):
        ''' Seems to be the case that ring is right channel 0,
                                       tip is  left channel 1'''
        self.pads.append(pad)
        if self._pad_count < self.channels:
            pad.link(self.queue[self._pad_count].get_static_pad('sink'))
            self.queue[self._pad_count].get_static_pad('src').link(
                self.fakesink[self._pad_count].get_static_pad('sink'))
            self._pad_count += 1
        else:
            log.debug('ignoring channels > %d' % self.channels)

    def set_handoff_signal(self, handoff_state):
        '''Sets whether the handoff signal would generate an interrupt
        or not'''
        for i in range(len(self.fakesink)):
            self.fakesink[i].set_property('signal-handoffs', handoff_state)

    def _new_buffer(self, buf, channel):
        ''' Use a new buffer '''
        if not self._dont_queue_the_buffer:
            self.callable1(buf, channel=channel)

    def on_buffer(self, element, data_buffer, pad, channel):
        '''The function that is called whenever new data is available
        This is the signal handler for the handoff signal'''
        temp_buffer = fromstring(data_buffer.extract_dup(
            0, data_buffer.get_size()), 'int16')
        if not self._dont_queue_the_buffer:
            self._new_buffer(temp_buffer, channel=channel)
        return False

    def set_freeze_the_display(self, freeze=False):
        ''' Useful when just the display is needed to be frozen, but
        logging should continue '''
        self._dont_queue_the_buffer = not freeze

    def get_freeze_the_display(self):
        '''Returns state of queueing the buffer'''
        return not self._dont_queue_the_buffer

    def start_sound_device(self):
        '''Start or Restart grabbing data from the audio capture'''
        Gst.Event.new_flush_start()
        self.pipeline.set_state(Gst.State.PLAYING)

    def stop_sound_device(self):
        '''Stop grabbing data from capture device'''
        Gst.Event.new_flush_stop(False)
        self.pipeline.set_state(Gst.State.NULL)

    def set_logging_params(self, start_stop=False, interval=0):
        ''' Configures for logging of data: starts or stops a session;
        and sets the logging interval. '''
        self.we_are_logging = start_stop
        self._logging_interval = interval
        if not start_stop:
            if self._logging_timer:
                self._logging_timer.cancel()
                self._logging_timer = None
                self._log_this_sample = False
                self._logging_counter = 0
        elif interval != 0:
            self._make_timer()
        self._busy = False

    def _sample_now(self):
        ''' Log the current sample now. This method is called from the
        _logging_timer object when the interval expires. '''
        self._log_this_sample = True
        self._make_timer()

    def _make_timer(self):
        ''' Create the next timer that will trigger data logging. '''
        self._logging_timer = Timer(self._logging_interval, self._sample_now)
        self._logging_timer.start()

    def set_sampling_rate(self, sr):
        ''' Sets the sampling rate of the logging device. Sampling
        rate must be given as an integer for example 16000 for setting
        16Khz sampling rate The sampling rate would be set in the
        device to the nearest available. '''
        self.pause_grabbing()
        caps_str = 'audio/x-raw-int,rate=%d,channels=%d,depth=16' % (
            sr, self.channels)
        self.caps1.set_property('caps', Gst.caps_from_string(caps_str))
        self.resume_grabbing()

    def set_callable1(self, callable1):
        '''Sets the callable to the drawing function for giving the
        data at the end of idle-add'''
        self.callable1 = callable1

    def get_sampling_rate(self):
        ''' Gets the sampling rate of the capture device '''
        return int(self.caps1.get_property('caps')[0]['rate'])

    def start_grabbing(self):
        '''Called right at the start of the Activity'''
        self.start_sound_device()
        self.set_handoff_signal(True)

    def pause_grabbing(self):
        '''When Activity goes into background'''
        if self.we_are_logging:
            log.debug('We are logging... will not pause grabbing.')
        else:
            log.debug('Pause grabbing.')
            self.save_state()
            self.stop_sound_device()
        return

    def resume_grabbing(self):
        '''When Activity becomes active after going to background'''
        if self.we_are_logging:
            log.debug('We are logging... already grabbing.')
        else:
            log.debug('Restore grabbing.')
            self.restore_state()
            self.start_sound_device()
            self.set_handoff_signal(True)
        return

    def stop_grabbing(self):
        '''Not used ???'''
        self.stop_sound_device()
        self.set_handoff_signal(False)

    def save_state(self):
        '''Saves the state of all audio controls'''
        self.master = self.get_master()
        self.bias = self.get_bias()
        self.dc_mode = self.get_dc_mode()
        self.capture_gain = self.get_capture_gain()
        self.mic_boost = self.get_mic_boost()

    def restore_state(self):
        '''Put back all audio control settings from the saved state'''
        self.set_master(self.master)
        self.set_bias(self.bias)
        self.stop_grabbing()
        if self.channels > 1:
            self._unlink_sink_queues()
        self.set_dc_mode(self.dc_mode)
        self.start_grabbing()
        self.set_capture_gain(self.capture_gain)
        self.set_mic_boost(self.mic_boost)

    def amixer_set(self, control, state):
        ''' Direct call to amixer for old systems. '''
        if state:
            output = check_output(
                ['amixer', 'set', "%s" % (control), 'unmute'],
                'Problem with amixer set "%s" unmute' % (control))
        else:
            output = check_output(
                ['amixer', 'set', "%s" % (control), 'mute'],
                'Problem with amixer set "%s" mute' % (control))

    def mute_master(self):
        '''Mutes the Master Control'''
        self.amixer_set('Master', False)

    def unmute_master(self):
        '''Unmutes the Master Control'''
        self.amixer_set('Master', True)

    def set_master(self, master_val):
        '''Sets the Master gain slider settings
        master_val must be given as an integer between 0 and 100 indicating the
        percentage of the slider to be set'''
        output = check_output(
            ['amixer', 'set', 'Master', "%d%s" % (master_val, '%')],
            'Problem with amixer set Master')

    def get_master(self):
        '''Gets the MIC gain slider settings. The value returned is an
        integer between 0 and 100 and is an indicative of the
        percentage 0 to 100%'''
        output = check_output(['amixer', 'get', 'Master'],
                              'amixer: Could not get Master volume')
        if output is None:
            return 100
        else:
            pos = find(output, 'Front Left:')
            if pos == -1:
                pos = find(output, 'Mono:')
            output = output[pos:]
            output = output[find(output, '[') + 1:]
            output = output[:find(output, '%]')]
            return int(output)

    def set_bias(self, bias_state=False):
        '''Enables / disables bias voltage.'''
        if self.parent.hw == XO1:
            self.amixer_set('MIC Bias Enable', bias_state)
        else:
            self.amixer_set('V_REFOUT Enable', bias_state)

    def get_bias(self):
        '''Check whether bias voltage is enabled.'''
        if self.parent.hw == XO1:
            control = 'MIC Bias Enable'
        else:
            control = 'V_REFOUT Enable'
        output = check_output(['amixer', 'get', control],
                              'amixer: Could not get mic bias voltage')
        if output is None:
            return False
        else:
            output = output[find(output, 'Mono:'):]
            output = output[find(output, '[') + 1:]
            output = output[:find(output, ']')]
            if output == 'on':
                return True
            return False

    def set_dc_mode(self, dc_mode=False):
        '''Sets the DC Mode Enable control
        pass False to mute and True to unmute'''
        self.amixer_set('DC Mode Enable', dc_mode)

    def get_dc_mode(self):
        '''Returns the setting of DC Mode Enable control
        i.e. True: Unmuted and False: Muted'''
        output = check_output(['amixer', 'get', "DC Mode Enable"],
                              'amixer: Could not get DC Mode')
        if output is None:
            return False
        else:
            output = output[find(output, 'Mono:'):]
            output = output[find(output, '[') + 1:]
            output = output[:find(output, ']')]
            if output == 'on':
                return True
            return False

    def set_mic_boost(self, mic_boost=False):
        '''Set Mic Boost.
        for analog mic boost: True = +20dB, False = 0dB
        for mic1 boost: True = 8, False = 0'''
        self.amixer_set('Mic Boost (+20dB)', mic_boost)

    def get_mic_boost(self):
        '''Return Mic Boost setting.
        for analog mic boost: True = +20dB, False = 0dB
        for mic1 boost: True = 8, False = 0'''
        output = check_output(['amixer', 'get', "Mic Boost (+20dB)"],
                              'amixer: Could not get mic boost')
        if output is None:
            return False
        else:
            output = output[find(output, 'Mono:'):]
            output = output[find(output, '[') + 1:]
            output = output[:find(output, ']')]
            if output == 'on':
                return True
            return False

    def set_capture_gain(self, capture_val):
        '''Sets the Capture gain slider settings capture_val must be
        given as an integer between 0 and 100 indicating the
        percentage of the slider to be set'''
        output = check_output(
            ['amixer', 'set', 'Capture', "%d%s" % (capture_val, '%')],
            'Problem with amixer set Capture')

    def get_capture_gain(self):
        '''Gets the Capture gain slider settings. The value returned
        is an integer between 0 and 100 and is an indicative of the
        percentage 0 to 100%'''
        output = check_output(['amixer', 'get', 'Capture'],
                              'amixer: Could not get Capture level')
        if output is None:
            return 100
        else:
            pos = find(output, 'Front Left:')
            if pos == -1:
                pos = find(output, 'Mono:')
            output = output[pos:]
            output = output[find(output, '[') + 1:]
            output = output[:find(output, '%]')]
            return int(output)

    def set_mic_gain(self, mic_val):
        '''Sets the MIC gain slider settings mic_val must be given as
        an integer between 0 and 100 indicating the percentage of the
        slider to be set'''
        output = check_output(
            ['amixer', 'set', 'Mic', "%d%s" % (mic_val, '%')],
            'Problem with amixer set Mic')

    def get_mic_gain(self):
        '''Gets the MIC gain slider settings. The value returned is an
        integer between 0 and 100 and is an indicative of the
        percentage 0 to 100%'''
        output = check_output(['amixer', 'get', 'Mic'],
                              'amixer: Could not get mic gain level')
        if output is None:
            return 100
        else:
            output = output[find(output, 'Mono:'):]
            output = output[find(output, '[') + 1:]
            output = output[:find(output, '%]')]
            return int(output)

    def _set_sensor_type(self, mode=None, bias=None, gain=None, boost=None):
        '''Helper to modify (some) of the sensor settings.'''

        if mode is not None:
            self.set_dc_mode(mode)

        if bias is not None:
            self.set_bias(bias)

        if gain is not None:
            self.set_capture_gain(gain)

        if boost is not None:
            self.set_mic_boost(boost)

        self.save_state()

    def _on_activity_quit(self):
        '''When Activity quits'''
        self.set_mic_boost(QUIT_MIC_BOOST)
        self.set_dc_mode(QUIT_DC_MODE_ENABLE)
        self.set_capture_gain(QUIT_CAPTURE_GAIN)
        self.set_bias(QUIT_BIAS)
        self.stop_sound_device()

    def on_activity_quit(self):
        AudioGrab._on_activity_quit(self)
        output = check_output(
            ['amixer', 'set', 'Analog Mic Boost', "62%"],
            'restore Analog Mic Boost')


def check_output(command, warning):
    ''' Workaround for old systems without subprocess.check_output'''
    if hasattr(subprocess, 'check_output'):
        try:
            output = subprocess.check_output(command)
        except subprocess.CalledProcessError:
            log.warning(warning)
            return None
    else:
        import commands

        cmd = ''
        for c in command:
            cmd += c
            cmd += ' '
        (status, output) = commands.getstatusoutput(cmd)
        if status != 0:
            log.warning(warning)
            return None
    return output



# utils.py - Helper functions for tis2000.py
# Copyright (C) 2010 Emiliano Pastorino <epastorino@plan.ceibal.edu.uy>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import logging


def find_device():
    """
    Search for devices.
    Return a device instance or None.
    """
    device = None
    path = os.path.dirname(os.path.abspath(__file__))
    for i in os.listdir(path):
        if not os.path.isdir(os.path.join(path, i)):
            try:
                _tempmod = __import__('rfid.%s' % i.split('.')[0], globals(),
                                      locals(), ['RFIDReader'], -1)
                devtemp = _tempmod.RFIDReader()
                if devtemp.get_present():
                    device = devtemp
            except Exception as e:
                # logging.error("FIND_DEVICE: %s: %s"%(i, e))
                pass
    if device is None:
        logging.debug("No RFID device found")
    return device


def strhex2bin(strhex):
    """
    Convert a string representing an hex value into a
    string representing the same value in binary format.
    """
    dic = {'0': "0000",
           '1': "0001",
           '2': "0010",
           '3': "0011",
           '4': "0100",
           '5': "0101",
           '6': "0110",
           '7': "0111",
           '8': "1000",
           '9': "1001",
           'A': "1010",
           'B': "1011",
           'C': "1100",
           'D': "1101",
           'E': "1110",
           'F': "1111"
           }
    binstr = ""
    for i in strhex:
        binstr = binstr + dic[i.upper()]
    return binstr


def strbin2dec(strbin):
    """
    Convert a string representing a binary value into a
    string representing the same value in decimal format.
    """
    strdec = "0"
    for i in range(1, strbin.__len__() + 1):
        strdec = str(int(strdec) + int(strbin[-i]) * int(pow(2, i - 1)))
    return strdec


def dec2bin(ndec):
    """
    Convert a decimal number into a string representing
    the same value in binary format.
    """
    if ndec < 1:
        return "0"
    binary = []
    while ndec != 0:
        binary.append(ndec % 2)
        ndec = ndec / 2
    strbin = ""
    binary.reverse()
    for i in binary:
        strbin = strbin + str(i)
    return strbin


def bin2hex(strbin):
    """
    Convert a string representing a binary number into a string
    representing the same value in hexadecimal format.
    """
    dic = {"0000": "0",
           "0001": "1",
           "0010": "2",
           "0011": "3",
           "0100": "4",
           "0101": "5",
           "0110": "6",
           "0111": "7",
           "1000": "8",
           "1001": "9",
           "1010": "A",
           "1011": "B",
           "1100": "C",
           "1101": "D",
           "1110": "E",
           "1111": "F"
           }
    while strbin.__len__() % 4 != 0:
        strbin = '0' + strbin
    strh = ""
    for i in range(0, strbin.__len__() / 4):
        strh = strh + dic[str(strbin[i * 4:i * 4 + 4])]
    return strh



# -*- coding: utf-8 -*-
# Copyright (c) 2011, 2012 Walter Bender

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

'''
This file contains the constants  that by-in-large determine the
behavior of Turtle Art.  Notably, the block palettes are defined
below.  If you want to add a new block to Turtle Art,  you could
simply add a block of code to this file or to
../plugins/turtle_blocks_extras/turtle_blocks_extras.py ,
which contains additional blocks.  (Even better,  write your own
plugin!!)


Adding a new palette is simply a matter of:

    palette = make_palette('mypalette',  # the name of your palette
                           colors=["#00FF00", "#00A000"],
                           help_string=_('Palette of my custom commands'))

For example,  if we want to add  a new turtle command,  'uturn',
we'd use the `add_block` method in the Palette class.

    palette.add_block('uturn',  # the name of your block
                      style='basic-style',  # the block style
                      label=_('u turn'),  # the label for the block
                      prim_name='uturn',  # code reference (see below)
                      help_string=_('turns the turtle 180 degrees'))

Next, you need to define what your block will do: def_prim takes
3 arguments: the primitive name, the number of arguments  ---  0
in this case  ---  and  a Primitive object.   A Primitive object
represents  the statement  to be  executed   when  the block  is
executed in Turtle Art. For the 'uturn' block, we would like the
statement to look roughly like this:

    Turtle.set_heading(plus(Turtle.get_heading(), 180))

Formally, a Primitive object consists of a function,  its return
type,  and descriptions  of its arguments and keyword arguments.
The return type  is not a Python type,   but a type  from Turtle
Art's internal type system.   All available types are defined as
constants in tatype.py .

In this case,  we know in advance  which arguments each function
gets,   so we can use  ConstantArg objects  as argument descrip-
tions. (For examples where the arguments come from other blocks,
please refer to ../doc/primitives-with-arguments.md .) Note that
Primitive objects  can be arguments  to other Primitive objects.
This leads to the following tree-like structure  for our 'uturn'
block:

    prim_uturn = Primitive(Turtle.set_heading,
        arg_descs=[ConstantArg(Primitive(
            Primitive.plus, return_type=TYPE_NUMBER,
                arg_descs=[ConstantArg(Primitive(
                    Turtle.get_heading, return_type=TYPE_NUMBER)),
                    ConstantArg(180)]))],
        call_afterwards=self.after_uturn)

    self.tw.lc.def_prim('uturn', 0, prim_uturn)

    # somewhere else in the same class:
    def after_uturn(self, value):
        if self.tw.lc.update_values:
            self.tw.lc.update_label_value('heading', value)

The `call_afterwards` attribute  is  a simple function   that is
called  just after executing  the block.   It is often used  for
updating GUI labels.

That's it. When you next run Turtle Art, you will have a 'uturn'
block on the 'mypalette' palette.

You will have to create icons  for the palette-selector buttons.
These are kept in the 'icons' subdirectory.  You need two icons:
mypaletteoff.svg and mypaletteon.svg,   where 'mypalette' is the
same string  as the entry  you used in instantiating the Palette
object.  Note  that the icons should be the same size (55x55) as
the others. (This is the default icon size for Sugar toolbars.)
'''

from time import time
from gettext import gettext as _

from .tapalette import (make_palette, define_logo_function)
from .talogo import primitive_dictionary
from .taconstants import (
    Color,
    CONSTANTS,
    DEFAULT_BACKGROUND_COLOR_SHADE_GRAY)
from .taprimitive import (ArgSlot, ConstantArg, or_, Primitive)
from .tatype import (TYPE_BOOL, TYPE_BOX, TYPE_CHAR, TYPE_COLOR, TYPE_FLOAT,
                    TYPE_INT, TYPE_NUMBER, TYPE_NUMERIC_STRING, TYPE_OBJECT,
                    TYPE_STRING, TYPE_VECTOR)
from .taturtle import Turtle


def _millisecond():
    ''' Current time in milliseconds '''
    return time() * 1000


class Palettes():

    ''' a class for creating the palettes of blocks '''

    def __init__(self, turtle_window):
        self.tw = turtle_window

        self.prim_cache = {
            "minus": Primitive(Primitive.minus,
                               return_type=TYPE_NUMBER,
                               arg_descs=[ArgSlot(TYPE_NUMBER)]),
            "ord": Primitive(ord,
                             return_type=TYPE_INT,
                             arg_descs=[ArgSlot(TYPE_CHAR)])
        }  # avoid several Primitives of the same function

        self._turtle_palette()

        self._pen_palette()

        self._color_palette()

        self._numbers_palette()

        self._flow_palette()

        self._blocks_palette()

    def make_trash_palette(self):
        self._trash_palette()

    # Palette definitions

    def _turtle_palette(self):
        ''' The basic Turtle Art turtle palette '''

        palette = make_palette('turtle',
                               colors=["#00FF00", "#00A000"],
                               help_string=_('Palette of turtle commands'),
                               translation=_('turtle'))

        palette.add_block('forward',
                          style='basic-style-1arg',
                          label=_('forward'),
                          prim_name='forward',
                          default=100,
                          logo_command='forward',
                          help_string=_('moves turtle forward'))
        self.tw.lc.def_prim(
            'forward', 1,
            Primitive(Turtle.forward,
                      arg_descs=[ArgSlot(TYPE_NUMBER)],
                      call_afterwards=self.after_move))

        palette.add_block('back',
                          style='basic-style-1arg',
                          label=_('back'),
                          prim_name='back',
                          default=100,
                          logo_command='back',
                          help_string=_('moves turtle backward'))
        self.tw.lc.def_prim(
            'back', 1,
            Primitive(Turtle.backward,
                      arg_descs=[ArgSlot(TYPE_NUMBER)],
                      call_afterwards=self.after_move))

        palette.add_block('clean',
                          style='basic-style-extended-vertical',
                          label=_('clean'),
                          prim_name='clean',
                          logo_command='clean',
                          help_string=_('clears the screen and reset the \
turtle'))
        self.tw.lc.def_prim('clean', 0,
                            Primitive(Primitive.group, arg_descs=[
                                ConstantArg([
                                    Primitive(self.tw.clear_plugins),
                                    Primitive(self.tw.lc.stop_playing_media),
                                    Primitive(self.tw.lc.reset_scale),
                                    # Primitive(self.tw.lc.reset_timer),
                                    Primitive(self.tw.lc.clear_value_blocks),
                                    Primitive(self.tw.canvas.clearscreen),
                                    Primitive(self.tw.lc.reset_internals),
                                    Primitive(self.tw.turtles.reset_turtles),
                                    Primitive(self.tw.lc.active_turtle)
                                ])]))

        palette.add_block('left',
                          style='basic-style-1arg',
                          label=_('left'),
                          prim_name='left',
                          default=90,
                          logo_command='left',
                          help_string=_('turns turtle counterclockwise (angle \
in degrees)'))
        self.tw.lc.def_prim(
            'left', 1,
            Primitive(Turtle.left,
                      arg_descs=[ArgSlot(TYPE_NUMBER)],
                      call_afterwards=self.after_right))

        palette.add_block('right',
                          style='basic-style-1arg',
                          label=_('right'),
                          prim_name='right',
                          default=90,
                          logo_command='right',
                          help_string=_('turns turtle clockwise (angle in \
degrees)'))
        self.tw.lc.def_prim(
            'right', 1,
            Primitive(Turtle.right,
                      arg_descs=[ArgSlot(TYPE_NUMBER)],
                      call_afterwards=self.after_right))

        palette.add_block('arc',
                          style='basic-style-2arg',
                          label=[_('arc'), _('angle'), _('radius')],
                          prim_name='arc',
                          default=[90, 100],
                          logo_command='taarc',
                          help_string=_('moves turtle along an arc'))
        self.tw.lc.def_prim(
            'arc', 2,
            Primitive(Turtle.arc,
                      arg_descs=[ArgSlot(TYPE_NUMBER),
                                 ArgSlot(TYPE_NUMBER)],
                      call_afterwards=self.after_arc))
        define_logo_function('taarc', 'to taarc :a :r\nrepeat round :a \
[right 1 forward (0.0175 * :r)]\nend\n')

        palette.add_block('setxy2',
                          style='basic-style-2arg',
                          label=[_('set xy'), _('x'), _('y')],
                          prim_name='setxy2',
                          logo_command='tasetxy',
                          default=[0, 0],
                          help_string=_('moves turtle to position xcor, ycor; \
(0, 0) is in the center of the screen.'))
        self.tw.lc.def_prim(
            'setxy2', 2,
            Primitive(Turtle.set_xy,
                      arg_descs=[ArgSlot(TYPE_NUMBER), ArgSlot(TYPE_NUMBER)],
                      call_afterwards=self.after_move))
        define_logo_function('tasetxy', 'to tasetxy :x :y\nsetxy :x :y\nend\n')

        palette.add_block('seth',
                          style='basic-style-1arg',
                          label=_('set heading'),
                          prim_name='seth',
                          default=0,
                          logo_command='seth',
                          help_string=_('sets the heading of the turtle (0 is \
towards the top of the screen.)'))
        self.tw.lc.def_prim(
            'seth', 1,
            Primitive(Turtle.set_heading,
                      arg_descs=[ArgSlot(TYPE_NUMBER)],
                      call_afterwards=lambda value: self.after_set(
                          'heading', value)))

        palette.add_block('xcor',
                          style='box-style',
                          label=_('xcor'),
                          help_string=_('holds current x-coordinate value of \
the turtle (can be used in place of a number block)'),
                          value_block=True,
                          prim_name='xcor',
                          logo_command='xcor')
        self.tw.lc.def_prim(
            'xcor', 0,
            Primitive(Primitive.divide, return_type=TYPE_FLOAT,
                      arg_descs=[ConstantArg(Primitive(Turtle.get_x)),
                                 ConstantArg(Primitive(
                                     self.tw.get_coord_scale))]))

        palette.add_block('ycor',
                          style='box-style',
                          label=_('ycor'),
                          help_string=_('holds current y-coordinate value of \
the turtle (can be used in place of a number block)'),
                          value_block=True,
                          prim_name='ycor',
                          logo_command='ycor')
        self.tw.lc.def_prim(
            'ycor', 0,
            Primitive(Primitive.divide, return_type=TYPE_FLOAT,
                      arg_descs=[ConstantArg(Primitive(Turtle.get_y)),
                                 ConstantArg(Primitive(
                                     self.tw.get_coord_scale))]))

        palette.add_block('heading',
                          style='box-style',
                          label=_('heading'),
                          help_string=_('holds current heading value of the \
turtle (can be used in place of a number block)'),
                          value_block=True,
                          prim_name='heading',
                          logo_command='heading')
        self.tw.lc.def_prim('heading', 0,
                            Primitive(
                                Turtle.get_heading, return_type=TYPE_NUMBER))

        palette.add_block('turtle-label',
                          hidden=True,
                          style='blank-style',
                          label=['turtle'])

        # Deprecated
        palette.add_block('setxy',
                          hidden=True,
                          style='basic-style-2arg',
                          label=[_('set xy'), _('x'), _('y')],
                          prim_name='setxy',
                          default=[0, 0],
                          logo_command='tasetxypenup',
                          help_string=_('moves turtle to position xcor, ycor; \
(0, 0) is in the center of the screen.'))
        self.tw.lc.def_prim(
            'setxy', 2,
            Primitive(Turtle.set_xy,
                      arg_descs=[ArgSlot(TYPE_NUMBER), ArgSlot(TYPE_NUMBER)],
                      kwarg_descs={'pendown': ConstantArg(False)},
                      call_afterwards=self.after_move))
        define_logo_function('tasetxypenup', 'to tasetxypenup :x :y\npenup\n\
setxy :x :y\npendown\nend\n')

    def _pen_palette(self):
        ''' The basic Turtle Art pen palette '''

        palette = make_palette('pen',
                               colors=["#00FFFF", "#00A0A0"],
                               help_string=_('Palette of pen commands'),
                               translation=_('pen'))

        palette.add_block('fillscreen',
                          hidden=True,
                          style='basic-style-2arg',
                          label=[_('fill screen'), _('color'), _('shade')],
                          prim_name='fillscreen',
                          default=DEFAULT_BACKGROUND_COLOR_SHADE_GRAY[0:2],
                          logo_command='tasetbackground',
                          help_string=_('fills the background with (color, \
shade)'))
        self.tw.lc.def_prim(
            'fillscreen', 2,
            Primitive(self.tw.canvas.fillscreen,
                      arg_descs=[ArgSlot(TYPE_NUMBER), ArgSlot(TYPE_NUMBER)]))

        palette.add_block('fillscreen2',
                          style='basic-style-3arg',
                          label=[_('fill screen') + '\n\n', _('color'),
                                 _('shade'), _('gray')],
                          prim_name='fillscreen2',
                          default=DEFAULT_BACKGROUND_COLOR_SHADE_GRAY,
                          logo_command='tasetbackground',
                          help_string=_('fills the background with (color, \
shade)'))
        self.tw.lc.def_prim(
            'fillscreen2', 3,
            Primitive(self.tw.canvas.fillscreen_with_gray,
                      arg_descs=[ArgSlot(TYPE_NUMBER), ArgSlot(TYPE_NUMBER),
                                 ArgSlot(TYPE_NUMBER)]))

        define_logo_function('tasetbackground', 'to tasetbackground :color \
:shade\ntasetshade :shade\nsetbackground :color\nend\n')

        palette.add_block('setcolor',
                          style='basic-style-1arg',
                          label=_('set color'),
                          prim_name='setcolor',
                          default=0,
                          logo_command='tasetpencolor',
                          help_string=_('sets color of the line drawn by the \
turtle'))
        self.tw.lc.def_prim(
            'setcolor', 1,
            Primitive(Turtle.set_color,
                      arg_descs=[or_(ArgSlot(TYPE_COLOR),
                                     ArgSlot(TYPE_NUMBER))],
                      call_afterwards=lambda value: self.after_set(
                          'color', value)))

        palette.add_block('setshade',
                          style='basic-style-1arg',
                          label=_('set shade'),
                          prim_name='setshade',
                          default=50,
                          logo_command='tasetshade',
                          help_string=_('sets shade of the line drawn by the \
turtle'))
        self.tw.lc.def_prim(
            'setshade', 1,
            Primitive(Turtle.set_shade,
                      arg_descs=[ArgSlot(TYPE_NUMBER)],
                      call_afterwards=lambda value: self.after_set(
                          'shade', value)))

        palette.add_block('setgray',
                          style='basic-style-1arg',
                          label=_('set gray'),
                          prim_name='setgray',
                          default=100,
                          help_string=_('sets gray level of the line drawn by \
the turtle'))
        self.tw.lc.def_prim(
            'setgray', 1,
            Primitive(Turtle.set_gray,
                      arg_descs=[ArgSlot(TYPE_NUMBER)],
                      call_afterwards=lambda value: self.after_set(
                          'gray', value)))

        palette.add_block('color',
                          style='box-style',
                          label=_('color'),
                          help_string=_('holds current pen color (can be used \
in place of a number block)'),
                          value_block=True,
                          prim_name='color',
                          logo_command='pencolor')
        self.tw.lc.def_prim('color', 0,
                            Primitive(
                                Turtle.get_color, return_type=TYPE_NUMBER))

        palette.add_block('shade',
                          style='box-style',
                          label=_('shade'),
                          help_string=_('holds current pen shade'),
                          value_block=True,
                          prim_name='shade',
                          logo_command=':shade')
        self.tw.lc.def_prim(
            'shade',
            0,
            Primitive(
                Turtle.get_shade,
                return_type=TYPE_NUMBER))

        palette.add_block('gray',
                          style='box-style',
                          label=_('gray'),
                          help_string=_('holds current gray level (can be \
used in place of a number block)'),
                          value_block=True,
                          prim_name='gray')
        self.tw.lc.def_prim('gray', 0,
                            Primitive(
                                Turtle.get_gray, return_type=TYPE_NUMBER))

        palette.add_block('penup',
                          style='basic-style-extended-vertical',
                          label=_('pen up'),
                          prim_name='penup',
                          logo_command='penup',
                          help_string=_('Turtle will not draw when moved.'))
        self.tw.lc.def_prim(
            'penup', 0,
            Primitive(Turtle.set_pen_state, arg_descs=[ConstantArg(False)]))

        palette.add_block('pendown',
                          style='basic-style-extended-vertical',
                          label=_('pen down'),
                          prim_name='pendown',
                          logo_command='pendown',
                          help_string=_('Turtle will draw when moved.'))
        self.tw.lc.def_prim(
            'pendown', 0,
            Primitive(Turtle.set_pen_state, arg_descs=[ConstantArg(True)]))

        palette.add_block('penstate',
                          style='boolean-block-style',
                          label=_('pen down?'),
                          prim_name='penstate',
                          help_string=_('returns True if pen is down'))
        self.tw.lc.def_prim(
            'penstate', 0,
            Primitive(Turtle.get_pen_state, return_type=TYPE_BOOL))

        palette.add_block('setpensize',
                          style='basic-style-1arg',
                          label=_('set pen size'),
                          prim_name='setpensize',
                          default=5,
                          logo_command='setpensize',
                          help_string=_('sets size of the line drawn by the \
turtle'))
        self.tw.lc.def_prim(
            'setpensize', 1,
            Primitive(Turtle.set_pen_size,
                      arg_descs=[ArgSlot(TYPE_NUMBER)],
                      call_afterwards=lambda val: self.after_set(
                          'pensize', val)))
        define_logo_function('tasetpensize',
                             'to tasetpensize :a\nsetpensize round :a\nend\n')

        palette.add_block('startfill',
                          style='basic-style-extended-vertical',
                          label=_('start fill'),
                          prim_name='startfill',
                          help_string=_('starts filled polygon (used with end \
fill block)'))
        self.tw.lc.def_prim('startfill', 0, Primitive(Turtle.start_fill))

        palette.add_block('stopfill',
                          style='basic-style-extended-vertical',
                          label=_('end fill'),
                          prim_name='stopfill',
                          help_string=_('completes filled polygon (used with \
start fill block)'))
        self.tw.lc.def_prim('stopfill', 0, Primitive(Turtle.stop_fill))

        palette.add_block('pensize',
                          style='box-style',
                          label=_('pen size'),
                          help_string=_('holds current pen size (can be used \
in place of a number block)'),
                          value_block=True,
                          prim_name='pensize',
                          logo_command='pensize')
        self.tw.lc.def_prim(
            'pensize', 0,
            Primitive(Turtle.get_pen_size, return_type=TYPE_NUMBER))
        define_logo_function('tapensize', 'to tapensize\noutput first round \
pensize\nend\n')

    def _rgb_converter(self, r, g, b):
        return float(self.tw.canvas.get_color_index(r, g, b))

    def _color_palette(self):
        ''' The basic Turtle Art color palette '''

        palette = make_palette('colors',
                               colors=["#00FFFF", "#00A0A0"],
                               help_string=_('Palette of pen colors'),
                               translation=_('colors'))

        color_names = ('red', 'orange', 'yellow', 'green', 'cyan', 'blue',
                       'purple', 'white', 'black')
        # Need to make sure color names are included in the PO files
        color_names_i18n = (_('red'), _('orange'), _('yellow'),  # noqa: F841
                            _('green'), _('cyan'), _('blue'),
                            _('purple'), _('white'), _('black'))
        for name in color_names:
            self._make_constant(palette, name, _(name), name)

        palette.add_block('RGB to color',
                          style='number-style-var-3arg',
                          label=[_('RGB to\n\ncolor'), _(
                              'red'), _('green'), _('blue')],
                          default=[0, 0, 0],
                          help_string=_('converter'),
                          prim_name='converter')

        self.tw.lc.def_prim(
            'converter',
            3,
            Primitive(
                self._rgb_converter,
                return_type=TYPE_FLOAT,
                arg_descs=[
                    ArgSlot(TYPE_INT),
                    ArgSlot(TYPE_INT),
                    ArgSlot(TYPE_INT)]))

        # In order to map Turtle Art colors to the standard UCB Logo palette,
        # we need to define a somewhat complex set of functions.
        define_logo_function('tacolor', '\
to tasetpalette :i :r :g :b :myshade \n\
make "s ((:myshade - 50) / 50) \n\
ifelse lessp :s 0 [ \n\
make "s (1 + (:s *0.8)) \n\
make "r (:r * :s) \n\
make "g (:g * :s) \n\
make "b (:b * :s) \n\
] [ \
make "s (:s * 0.9) \n\
make "r (:r + ((99-:r) * :s)) \n\
make "g (:g + ((99-:g) * :s)) \n\
make "b (:b + ((99-:b) * :s)) \n\
] \
setpalette :i (list :r :g :b) \n\
end \n\
\
to rgb :myi :mycolors :myshade \n\
make "myr first :mycolors \n\
make "mycolors butfirst :mycolors \n\
make "myg first :mycolors \n\
make "mycolors butfirst :mycolors \n\
make "myb first :mycolors \n\
make "mycolors butfirst :mycolors \n\
tasetpalette :myi :myr :myg :myb :myshade \n\
output :mycolors \n\
end \n\
\
to processcolor :mycolors :myshade \n\
if emptyp :mycolors [stop] \n\
make "i :i + 1 \n\
processcolor (rgb :i :mycolors :myshade) :myshade \n\
end \n\
\
to tasetshade :shade \n\
make "myshade modulo :shade 200 \n\
if greaterp :myshade 99 [make "myshade (199-:myshade)] \n\
make "i 7 \n\
make "mycolors :colors \n\
processcolor :mycolors :myshade \n\
end \n\
\
to tasetpencolor :c \n\
make "color (modulo (round :c) 100) \n\
setpencolor :color + 8 \n\
end \n\
\
make "colors [ \
99  0  0 99  5  0 99 10  0 99 15  0 99 20  0 \
99 25  0 99 30  0 99 35  0 99 40  0 99 45  0 \
99 50  0 99 55  0 99 60  0 99 65  0 99 70  0 \
99 75  0 99 80  0 99 85  0 99 90  0 99 95  0 \
99 99  0 90 99  0 80 99  0 70 99  0 60 99  0 \
50 99  0 40 99  0 30 99  0 20 99  0 10 99  0 \
 0 99  0  0 99  5  0 99 10  0 99 15  0 99 20 \
 0 99 25  0 99 30  0 99 35  0 99 40  0 99 45 \
 0 99 50  0 99 55  0 99 60  0 99 65  0 99 70 \
 0 99 75  0 99 80  0 99 85  0 99 90  0 99 95 \
 0 99 99  0 95 99  0 90 99  0 85 99  0 80 99 \
 0 75 99  0 70 99  0 65 99  0 60 99  0 55 99 \
 0 50 99  0 45 99  0 40 99  0 35 99  0 30 99 \
 0 25 99  0 20 99  0 15 99  0 10 99  0  5 99 \
 0  0 99  5  0 99 10  0 99 15  0 99 20  0 99 \
25  0 99 30  0 99 35  0 99 40  0 99 45  0 99 \
50  0 99 55  0 99 60  0 99 65  0 99 70  0 99 \
75  0 99 80  0 99 85  0 99 90  0 99 95  0 99 \
99  0 99 99  0 90 99  0 80 99  0 70 99  0 60 \
99  0 50 99  0 40 99  0 30 99  0 20 99  0 10] \n\
make "shade  50 \n\
tasetshade :shade \n')

    def _numbers_palette(self):
        ''' The basic Turtle Art numbers palette '''

        palette = make_palette('numbers',
                               colors=["#FF00FF", "#A000A0"],
                               help_string=_('Palette of numeric operators'),
                               translation=_('numbers'))

        palette.add_block('plus2',
                          style='number-style',
                          label='+',
                          special_name=_('plus'),
                          string_or_number=True,
                          prim_name='plus',
                          logo_command='sum',
                          help_string=_('adds two alphanumeric inputs'))
        self.tw.lc.def_prim(
            'plus', 2,
            # add up two numbers ...
            or_(Primitive(Primitive.plus, return_type=TYPE_NUMBER,
                          arg_descs=[ArgSlot(TYPE_NUMBER),
                                     ArgSlot(TYPE_NUMBER)]),
                # ... or concatenate two strings
                Primitive(Primitive.plus, return_type=TYPE_STRING,
                          arg_descs=[ArgSlot(TYPE_STRING),
                                     ArgSlot(TYPE_STRING)]),
                # ... or add two vectors
                Primitive(Primitive.plus, return_type=TYPE_VECTOR,
                          arg_descs=[ArgSlot(TYPE_VECTOR),
                                     ArgSlot(TYPE_VECTOR)])))

        palette.add_block('minus2',
                          style='number-style-porch',
                          label='        –',
                          special_name=_('minus'),
                          prim_name='minus',
                          logo_command='taminus',
                          help_string=_('subtracts bottom numeric input from \
top numeric input'))
        self.tw.lc.def_prim(
            'minus', 2,
            or_(Primitive(Primitive.minus, return_type=TYPE_NUMBER,
                          arg_descs=[ArgSlot(TYPE_NUMBER),
                                     ArgSlot(TYPE_NUMBER)]),
                # ... or add two vectors
                Primitive(Primitive.minus, return_type=TYPE_VECTOR,
                          arg_descs=[ArgSlot(TYPE_VECTOR),
                                     ArgSlot(TYPE_VECTOR)])))

        define_logo_function('taminus', 'to taminus :y :x\noutput sum :x \
minus :y\nend\n')

        palette.add_block('product2',
                          style='number-style',
                          label='×',
                          special_name=_('multiply'),
                          prim_name='product',
                          logo_command='product',
                          help_string=_('multiplies two numeric inputs'))
        self.tw.lc.def_prim(
            'product', 2,
            or_(Primitive(Primitive.multiply, return_type=TYPE_NUMBER,
                          arg_descs=[ArgSlot(TYPE_NUMBER),
                                     ArgSlot(TYPE_NUMBER)]),
                Primitive(Primitive.multiply, return_type=TYPE_VECTOR,
                          arg_descs=[ArgSlot(TYPE_VECTOR),
                                     ArgSlot(TYPE_NUMBER)]),
                Primitive(Primitive.multiply, return_type=TYPE_VECTOR,
                          arg_descs=[ArgSlot(TYPE_NUMBER),
                                     ArgSlot(TYPE_VECTOR)])))

        palette.add_block('division2',
                          style='number-style-porch',
                          label='        /',
                          special_name=_('divide'),
                          prim_name='division',
                          logo_command='quotient',
                          help_string=_('divides top numeric input \
(numerator) by bottom numeric input (denominator)'))
        self.tw.lc.def_prim(
            'division', 2,
            or_(Primitive(Primitive.divide, return_type=TYPE_NUMBER,
                          arg_descs=[ArgSlot(TYPE_NUMBER),
                                     ArgSlot(TYPE_NUMBER)]),
                Primitive(Primitive.divide, return_type=TYPE_VECTOR,
                          arg_descs=[ArgSlot(TYPE_VECTOR),
                                     ArgSlot(TYPE_NUMBER)]),
                Primitive(Primitive.divide, return_type=TYPE_VECTOR,
                          arg_descs=[ArgSlot(TYPE_NUMBER),
                                     ArgSlot(TYPE_VECTOR)])))

        palette.add_block('identity2',
                          style='number-style-1arg',
                          label='←',
                          special_name=_('identity'),
                          prim_name='id',
                          help_string=_('identity operator used for extending \
blocks'))
        self.tw.lc.def_prim(
            'id', 1,
            # preserve the Type of the argument: try less general types first
            or_(Primitive(Primitive.identity, return_type=TYPE_NUMERIC_STRING,
                          arg_descs=[ArgSlot(TYPE_NUMERIC_STRING)]),
                Primitive(Primitive.identity, return_type=TYPE_CHAR,
                          arg_descs=[ArgSlot(TYPE_CHAR)]),
                Primitive(Primitive.identity, return_type=TYPE_COLOR,
                          arg_descs=[ArgSlot(TYPE_COLOR)]),
                Primitive(Primitive.identity, return_type=TYPE_FLOAT,
                          arg_descs=[ArgSlot(TYPE_FLOAT)]),
                Primitive(Primitive.identity, return_type=TYPE_INT,
                          arg_descs=[ArgSlot(TYPE_INT)]),
                Primitive(Primitive.identity, return_type=TYPE_NUMBER,
                          arg_descs=[ArgSlot(TYPE_NUMBER)]),
                Primitive(Primitive.identity, return_type=TYPE_STRING,
                          arg_descs=[ArgSlot(TYPE_STRING)]),
                Primitive(Primitive.identity, return_type=TYPE_VECTOR,
                          arg_descs=[ArgSlot(TYPE_VECTOR)]),
                Primitive(Primitive.identity, return_type=TYPE_OBJECT,
                          arg_descs=[ArgSlot(TYPE_OBJECT)])))

        palette.add_block('remainder2',
                          style='number-style-porch',
                          label=_('mod'),
                          special_name=_('mod'),
                          prim_name='remainder',
                          logo_command='remainder',
                          help_string=_('modular (remainder) operator'))
        self.tw.lc.def_prim(
            'remainder', 2,
            Primitive(Primitive.modulo, return_type=TYPE_NUMBER,
                      arg_descs=[ArgSlot(TYPE_NUMBER), ArgSlot(TYPE_NUMBER)]))

        palette.add_block('sqrt',
                          style='number-style-1arg',
                          label=_('√'),
                          special_name=_('square root'),
                          prim_name='sqrt',
                          logo_command='tasqrt',
                          help_string=_('calculates square root'))
        self.tw.lc.def_prim(
            'sqrt', 1,
            Primitive(Primitive.square_root, return_type=TYPE_FLOAT,
                      arg_descs=[ArgSlot(TYPE_NUMBER)]))

        palette.add_block('random',
                          style='number-style-block',
                          label=[_('random'), _('min'), _('max')],
                          default=[0, 100],
                          prim_name='random',
                          logo_command='tarandom',
                          help_string=_('returns random number between \
minimum (top) and maximum (bottom) values'))

        self.tw.lc.def_prim(
            'random', 2,
            or_(  # random character ...
                Primitive(Primitive.random_char, return_type=TYPE_CHAR,
                          arg_descs=[
                              ArgSlot(TYPE_INT,
                                      wrapper=self.prim_cache["ord"]),
                              ArgSlot(TYPE_INT,
                                      wrapper=self.prim_cache["ord"])]),
                # ... or random number
                Primitive(Primitive.random_int, return_type=TYPE_INT,
                          arg_descs=[ArgSlot(TYPE_INT), ArgSlot(TYPE_INT)])))
        define_logo_function('tarandom', 'to tarandom :min :max\n \
output (random (:max - :min)) + :min\nend\n')

        palette.add_block('number',
                          style='box-style',
                          label='100',
                          default=100,
                          special_name=_('number'),
                          help_string=_('used as numeric input in mathematic \
operators'))

        palette.add_block('greater2',
                          style='compare-porch-style',
                          label='    >',
                          string_or_number=True,
                          special_name=_('greater than'),
                          prim_name='greater?',
                          logo_command='greater?',
                          help_string=_('logical greater-than operator'))
        self.tw.lc.def_prim(
            'greater?', 2,
            Primitive(Primitive.greater, return_type=TYPE_BOOL,
                      arg_descs=or_([ArgSlot(TYPE_COLOR),
                                     ArgSlot(TYPE_COLOR)],
                                    [ArgSlot(TYPE_NUMBER),
                                     ArgSlot(TYPE_NUMBER)],
                                    [ArgSlot(TYPE_STRING),
                                     ArgSlot(TYPE_STRING)],
                                    [ArgSlot(TYPE_OBJECT),
                                     ArgSlot(TYPE_OBJECT)])))

        palette.add_block('less2',
                          style='compare-porch-style',
                          label='    <',
                          special_name=_('less than'),
                          string_or_number=True,
                          prim_name='less?',
                          logo_command='less?',
                          help_string=_('logical less-than operator'))
        self.tw.lc.def_prim(
            'less?', 2,
            Primitive(Primitive.less, return_type=TYPE_BOOL,
                      arg_descs=or_([ArgSlot(TYPE_COLOR),
                                     ArgSlot(TYPE_COLOR)],
                                    [ArgSlot(TYPE_NUMBER),
                                     ArgSlot(TYPE_NUMBER)],
                                    [ArgSlot(TYPE_STRING),
                                     ArgSlot(TYPE_STRING)],
                                    [ArgSlot(TYPE_OBJECT),
                                     ArgSlot(TYPE_OBJECT)])))

        palette.add_block('equal2',
                          style='compare-style',
                          label='=',
                          special_name=_('equal'),
                          string_or_number=True,
                          prim_name='equal?',
                          logo_command='equal?',
                          help_string=_('logical equal-to operator'))
        self.tw.lc.def_prim(
            'equal?', 2,
            Primitive(Primitive.equals, return_type=TYPE_BOOL,
                      arg_descs=or_([ArgSlot(TYPE_COLOR),
                                     ArgSlot(TYPE_COLOR)],
                                    [ArgSlot(TYPE_NUMBER),
                                     ArgSlot(TYPE_NUMBER)],
                                    [ArgSlot(TYPE_STRING),
                                     ArgSlot(TYPE_STRING)],
                                    [ArgSlot(TYPE_OBJECT),
                                     ArgSlot(TYPE_OBJECT)])))

        palette.add_block('not',
                          style='not-style',
                          label=_('not'),
                          prim_name='not',
                          logo_command='not',
                          help_string=_('logical NOT operator'))
        self.tw.lc.def_prim(
            'not', 1,
            Primitive(Primitive.not_, return_type=TYPE_BOOL,
                      arg_descs=[ArgSlot(TYPE_BOOL)]))

        palette.add_block('and2',
                          style='boolean-style',
                          label=_('and'),
                          prim_name='and',
                          logo_command='and',
                          special_name=_('and'),
                          help_string=_('logical AND operator'))
        self.tw.lc.def_prim(
            'and', 2,
            Primitive(Primitive.and_, return_type=TYPE_BOOL,
                      arg_descs=[ArgSlot(TYPE_BOOL), ArgSlot(TYPE_BOOL)]))

        palette.add_block('or2',
                          style='boolean-style',
                          label=_('or'),
                          prim_name='or',
                          logo_command='or',
                          special_name=_('or'),
                          help_string=_('logical OR operator'))
        self.tw.lc.def_prim(
            'or', 2,
            Primitive(Primitive.or_, return_type=TYPE_BOOL,
                      arg_descs=[ArgSlot(TYPE_BOOL), ArgSlot(TYPE_BOOL)]))

    def _flow_palette(self):
        ''' The basic Turtle Art flow palette '''

        palette = make_palette('flow',
                               colors=["#FFC000", "#A08000"],
                               help_string=_('Palette of flow operators'),
                               translation=_('flow'))

        palette.add_block('wait',
                          style='basic-style-1arg',
                          label=_('wait'),
                          prim_name='wait',
                          default=1,
                          logo_command='wait',
                          help_string=_('pauses program execution a specified \
number of seconds'))
        self.tw.lc.def_prim(
            'wait', 1,
            Primitive(self.tw.lc.prim_wait, arg_descs=[ArgSlot(TYPE_NUMBER)]),
            True)

        palette.add_block('forever',
                          style='clamp-style',
                          label=_('forever'),
                          prim_name='forever',
                          default=[None],
                          logo_command='forever',
                          help_string=_('loops forever'))
        self.tw.lc.def_prim(
            'forever', 1,
            Primitive(self.tw.lc.prim_loop, arg_descs=[ConstantArg(Primitive(
                Primitive.controller_forever)), ArgSlot(TYPE_OBJECT,
                                                        call_arg=False)]),
            True)

        palette.add_block('repeat',
                          style='clamp-style-1arg',
                          label=_('repeat'),
                          prim_name='repeat',
                          default=[4, None],
                          logo_command='repeat',
                          special_name=_('repeat'),
                          help_string=_('loops specified number of times'))
        self.tw.lc.def_prim(
            'repeat', 2,
            Primitive(self.tw.lc.prim_loop,
                      arg_descs=[ArgSlot(
                          TYPE_OBJECT,
                          wrapper=Primitive(Primitive.controller_repeat,
                                            arg_descs=[ArgSlot(TYPE_INT)])),
                                 ArgSlot(TYPE_OBJECT, call_arg=False)]),
            True)

        palette.add_block('if',
                          style='clamp-style-boolean',
                          label=[_('if'), _('then')],
                          prim_name='if',
                          default=[None, None],
                          special_name=_('if then'),
                          logo_command='if',
                          help_string=_('if-then operator that uses boolean \
operators from Numbers palette'))
        self.tw.lc.def_prim(
            'if', 2,
            Primitive(self.tw.lc.prim_if,
                      arg_descs=[ArgSlot(TYPE_BOOL), ArgSlot(TYPE_OBJECT)]),
            True)

        palette.add_block('ifelse',
                          hidden=True,  # Too big to fit palette
                          style='clamp-style-else',
                          label=[_('if'), _('then'), _('else')],
                          prim_name='ifelse',
                          default=[None, None, None],
                          logo_command='ifelse',
                          special_name=_('if then else'),
                          help_string=_('if-then-else operator that uses \
boolean operators from Numbers palette'))
        self.tw.lc.def_prim(
            'ifelse', 3,
            Primitive(self.tw.lc.prim_ifelse,
                      arg_descs=[ArgSlot(TYPE_BOOL), ArgSlot(TYPE_OBJECT),
                                 ArgSlot(TYPE_OBJECT)]),
            True)

        # macro
        palette.add_block('ifthenelse',
                          style='basic-style-extended-vertical',
                          label=_('if then else'),
                          help_string=_('if-then-else operator that uses \
boolean operators from Numbers palette'))

        # Deprecated
        palette.add_block('hspace',
                          style='flow-style-tail',
                          hidden=True,
                          label='',
                          prim_name='nop',
                          special_name=_('horizontal space'),
                          help_string=_('jogs stack right'))
        self.tw.lc.def_prim(
            'nop', 0,
            Primitive(Primitive.do_nothing, export_me=False))

        palette.add_block('vspace',
                          style='basic-style-extended-vertical',
                          label='',
                          prim_name='nop',
                          special_name=_('vertical space'),
                          help_string=_('jogs stack down'))
        self.tw.lc.def_prim(
            'nop', 0,
            Primitive(Primitive.do_nothing, export_me=False))

        palette.add_block('stopstack',
                          style='basic-style-tail',
                          label=_('stop action'),
                          prim_name='stopstack',
                          logo_command='stop',
                          help_string=_('stops current action'))
        self.tw.lc.def_prim(
            'stopstack', 0,
            Primitive(self.tw.lc.prim_stop_stack))

    def _blocks_palette(self):
        ''' The basic Turtle Art blocks palette '''

        palette = make_palette('blocks',
                               colors=["#FFFF00", "#A0A000"],
                               help_string=_('Palette of variable blocks'),
                               translation=_('blocks'))

        palette.add_block('start',
                          style='clamp-style-hat',
                          label=_('start'),
                          prim_name='start',
                          logo_command='to start\n',
                          help_string=_('connects action to toolbar run \
buttons'))
        self.tw.lc.def_prim(
            'start', 0,
            Primitive(Primitive.group, arg_descs=[ConstantArg([
                Primitive(self.tw.lc.prim_start,
                          export_me=False),
                Primitive(self.tw.lc.prim_define_stack,
                          arg_descs=[ConstantArg('start')])])]))

        palette.add_block('hat',
                          style='clamp-style-hat-1arg',
                          label=_('action'),
                          prim_name='nop3',
                          string_or_number=True,
                          default=_('action'),
                          logo_command='to action',
                          help_string=_('top of nameable action stack'))
        self.tw.lc.def_prim(
            'nop3', 1,
            Primitive(self.tw.lc.prim_define_stack,
                      arg_descs=[ArgSlot(TYPE_OBJECT)]))

        primitive_dictionary['stack'] = Primitive(
            self.tw.lc.prim_invoke_stack,
            arg_descs=[ArgSlot(TYPE_OBJECT)])
        palette.add_block('stack',
                          style='basic-style-1arg',
                          label=_('action'),
                          string_or_number=True,
                          prim_name='stack',
                          logo_command='action',
                          default=_('action'),
                          help_string=_('invokes named action stack'))
        self.tw.lc.def_prim('stack', 1,
                            primitive_dictionary['stack'], True)

        palette.add_block('storeinbox1',
                          hidden=True,
                          style='basic-style-1arg',
                          label=_('store in box 1'),
                          prim_name='storeinbox1',
                          default=100,
                          string_or_number=True,
                          logo_command='make "box1',
                          help_string=_('stores numeric value in Variable 1'))
        self.tw.lc.def_prim(
            'storeinbox1', 1,
            Primitive(self.tw.lc.prim_set_box,
                      arg_descs=[ConstantArg('box1'), ArgSlot(TYPE_OBJECT)]))

        palette.add_block('storeinbox2',
                          hidden=True,
                          style='basic-style-1arg',
                          label=_('store in box 2'),
                          prim_name='storeinbox2',
                          default=100,
                          string_or_number=True,
                          logo_command='make "box2',
                          help_string=_('stores numeric value in Variable 2'))
        self.tw.lc.def_prim(
            'storeinbox2', 1,
            Primitive(self.tw.lc.prim_set_box,
                      arg_descs=[ConstantArg('box2'), ArgSlot(TYPE_OBJECT)]))

        palette.add_block('box1',
                          hidden=True,
                          style='box-style',
                          label=_('box 1'),
                          prim_name='box1',
                          logo_command=':box1',
                          help_string=_('Variable 1 (numeric value)'),
                          value_block=True)
        self.tw.lc.def_prim(
            'box1', 0,
            Primitive(self.tw.lc.prim_get_box, return_type=TYPE_BOX,
                      arg_descs=[ConstantArg('box1')]))

        palette.add_block('box2',
                          hidden=True,
                          style='box-style',
                          label=_('box 2'),
                          prim_name='box2',
                          logo_command=':box2',
                          help_string=_('Variable 2 (numeric value)'),
                          value_block=True)
        self.tw.lc.def_prim(
            'box2', 0,
            Primitive(self.tw.lc.prim_get_box, return_type=TYPE_BOX,
                      arg_descs=[ConstantArg('box2')]))

        primitive_dictionary['setbox'] = Primitive(
            self.tw.lc.prim_set_box,
            arg_descs=[ArgSlot(TYPE_OBJECT), ArgSlot(TYPE_OBJECT)])
        palette.add_block('storein',
                          style='basic-style-2arg',
                          label=[_('store in'), _('box'), _('value')],
                          string_or_number=True,
                          prim_name='storeinbox',
                          logo_command='storeinbox',
                          default=[_('my box'), 100],
                          help_string=_('stores numeric value in named \
variable'))
        self.tw.lc.def_prim('storeinbox', 2, primitive_dictionary['setbox'])

        primitive_dictionary['box'] = Primitive(
            self.tw.lc.prim_get_box,
            return_type=TYPE_BOX,
            arg_descs=[ArgSlot(TYPE_OBJECT)])
        palette.add_block('box',
                          style='number-style-1strarg',
                          hidden=True,
                          label=_('box'),
                          string_or_number=True,
                          prim_name='box',
                          default=_('my box'),
                          logo_command='box',
                          value_block=True,
                          help_string=_('named variable (numeric value)'))
        self.tw.lc.def_prim('box', 1, primitive_dictionary['box'])

        palette.add_block('hat1',
                          hidden=True,
                          style='basic-style-head',
                          label=_('action 1'),
                          prim_name='nop1',
                          logo_command='to stack1\n',
                          help_string=_('top of Action 1 stack'))
        self.tw.lc.def_prim(
            'nop1', 0,
            Primitive(self.tw.lc.prim_define_stack,
                      arg_descs=[ConstantArg('stack1')]))

        palette.add_block('hat2',
                          hidden=True,
                          style='basic-style-head',
                          label=_('action 2'),
                          prim_name='nop2',
                          logo_command='to stack2\n',
                          help_string=_('top of Action 2 stack'))
        self.tw.lc.def_prim(
            'nop2', 0,
            Primitive(self.tw.lc.prim_define_stack,
                      arg_descs=[ConstantArg('stack2')]))

        palette.add_block('stack1',
                          hidden=True,
                          style='basic-style-extended-vertical',
                          label=_('action 1'),
                          prim_name='stack1',
                          logo_command='stack1',
                          help_string=_('invokes Action 1 stack'))
        self.tw.lc.def_prim(
            'stack1', 0,
            Primitive(self.tw.lc.prim_invoke_stack,
                      arg_descs=[ConstantArg('stack1')]),
            True)

        palette.add_block('stack2',
                          hidden=True,
                          style='basic-style-extended-vertical',
                          label=_('action 2'),
                          prim_name='stack2',
                          logo_command='stack2',
                          help_string=_('invokes Action 2 stack'))
        self.tw.lc.def_prim(
            'stack2', 0,
            Primitive(self.tw.lc.prim_invoke_stack,
                      arg_descs=[ConstantArg('stack2')]),
            True)

        primitive_dictionary['returnstack'] = Primitive(
            self.tw.lc.prim_invoke_return_stack,
            return_type=TYPE_OBJECT,
            arg_descs=[ArgSlot(TYPE_OBJECT)])
        palette.add_block('returnstack',
                          # hidden=True,
                          style='number-style-1arg',
                          label=_('action'),
                          string_or_number=True,
                          prim_name='returnstack',
                          logo_command='action',
                          default=_('action'),
                          help_string=_(
                              'invokes named action stack and returns value'))
        self.tw.lc.def_prim('returnstack', 1,
                            primitive_dictionary['returnstack'], True)

        palette.add_block('return',
                          # hidden=True,
                          style='basic-style-1arg',
                          label=_('return'),
                          prim_name='return',
                          logo_command='stop',
                          help_string=_('returns a value'))
        self.tw.lc.def_prim(
            'return', 1,
            Primitive(self.tw.lc.prim_return,
                      arg_descs=[ArgSlot(TYPE_OBJECT)]))

    def _trash_palette(self):
        ''' The basic Turtle Art turtle palette '''

        palette = make_palette('trash',
                               colors=["#FFFF00", "#A0A000"],
                               help_string=_('trash'),
                               translation=_('trash'))

        palette.add_block('empty',
                          style='blank-style',
                          label=_('empty trash'),
                          help_string=_('permanently deletes items in trash'))

        palette.add_block('restoreall',
                          style='blank-style',
                          label=_('restore all'),
                          help_string=_('restore all blocks from trash'))

        palette.add_block('trashall',
                          style='blank-style',
                          label=_('clear all'),
                          help_string=_('move all blocks to trash'))

    # Callbacks to update labels after executing a block

    def after_arc(self, *ignored_args):
        if self.tw.lc.update_values:
            self.tw.lc.update_label_value(
                'xcor',
                self.tw.turtles.get_active_turtle().get_xy()[0] /
                self.tw.coord_scale)
            self.tw.lc.update_label_value(
                'ycor',
                self.tw.turtles.get_active_turtle().get_xy()[1] /
                self.tw.coord_scale)
            self.tw.lc.update_label_value(
                'heading',
                self.tw.turtles.get_active_turtle().get_heading())

    def after_move(self, *ignored_args, **ignored_kwargs):
        ''' Update labels after moving the turtle '''
        if self.tw.lc.update_values:
            self.tw.lc.update_label_value(
                'xcor',
                self.tw.turtles.get_active_turtle().get_xy()[0] /
                self.tw.coord_scale)
            self.tw.lc.update_label_value(
                'ycor',
                self.tw.turtles.get_active_turtle().get_xy()[1] /
                self.tw.coord_scale)

    def after_right(self, *ignored_args):
        if self.tw.lc.update_values:
            self.tw.lc.update_label_value(
                'heading',
                self.tw.turtles.get_active_turtle().get_heading())

    def after_set(self, name, value=None):
        ''' Update the associated value blocks '''
        if value is not None:
            if self.tw.lc.update_values:
                self.tw.lc.update_label_value(name, value)

    # Utilities

    def _make_constant(self, palette, block_name, label, constant_key):
        ''' Factory for constant blocks '''
        constant = CONSTANTS[constant_key]
        if isinstance(constant, Color):
            if constant.color is not None:
                logo_command = str(constant.color)
            else:
                # Black or White
                logo_command = '0 tasetshade %d' % (constant.shade)
            return_type = TYPE_COLOR
        else:
            logo_command = constant
            return_type = TYPE_NUMBER
        palette.add_block(block_name,
                          style='box-style',
                          label=label,
                          prim_name=block_name,
                          logo_command=logo_command)
        self.tw.lc.def_prim(block_name, 0,
                            Primitive(CONSTANTS.get, return_type=return_type,
                                      arg_descs=[ConstantArg(constant_key)]))



# -*- coding: utf-8 -*-
# Copyright (c) 2008, Armin Ronacher
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
"""
    codegen
    ~~~~~~~

    Extension to ast that allow ast -> python code generation.

    :copyright: Copyright 2008 by Armin Ronacher.
    :license: BSD.

    Modified by Marion Zepf.
"""
from ast import *


def to_source(node, indent_with=' ' * 4, add_line_information=False):
    """This function can convert a node tree back into python sourcecode.
    This is useful for debugging purposes, especially if you're dealing with
    custom asts not generated by python itself.

    It could be that the sourcecode is evaluable when the AST itself is not
    compilable / evaluable.  The reason for this is that the AST contains some
    more data than regular sourcecode does, which is dropped during
    conversion.

    Each level of indentation is replaced with `indent_with`.  Per default this
    parameter is equal to four spaces as suggested by PEP 8, but it might be
    adjusted to match the application's styleguide.

    If `add_line_information` is set to `True` comments for the line numbers
    of the nodes are added to the output.  This can be used to spot wrong line
    number information of statement nodes.
    """
    generator = SourceGenerator(indent_with, add_line_information)
    generator.visit(node)
    return ''.join(generator.result)


class SourceGenerator(NodeVisitor):

    """This visitor is able to transform a well formed syntax tree into python
    sourcecode.  For more details have a look at the docstring of the
    `node_to_source` function.
    """

    UNARYOP_SYMBOLS = {Invert: "~", Not: "not", UAdd: "+", USub: "-"}
    # TODO use parentheses around expressions only where necessary
    BINOP_SYMBOLS = {Add: "+", Sub: "-", Mult: "*", Div: "/", Mod: "%",
                     LShift: "<<", RShift: ">>", BitOr: "|", BitXor: "^",
                     BitAnd: "&", FloorDiv: "//", Pow: "**"}
    BOOLOP_SYMBOLS = {And: "and", Or: "or"}
    CMPOP_SYMBOLS = {Eq: "==", NotEq: "!=", Lt: "<", LtE: "<=", Gt: ">",
                     GtE: ">=", Is: "is", IsNot: "is not", In: "in",
                     NotIn: "not in"}

    def __init__(self, indent_with, add_line_information=False):
        self.result = []
        self.indent_with = indent_with
        self.add_line_information = add_line_information
        self.indentation = 0
        self.new_lines = 0

    def write(self, x):
        if self.new_lines:
            if self.result:
                self.result.append('\n' * self.new_lines)
            self.result.append(self.indent_with * self.indentation)
            self.new_lines = 0
        self.result.append(x)

    def newline(self, node=None, extra=0):
        self.new_lines = max(self.new_lines, 1 + extra)
        if node is not None and self.add_line_information:
            self.write('# line: %s' % node.lineno)
            self.new_lines = 1

    def body(self, statements, do_indent=True):
        if do_indent:
            self.indentation += 1
        for stmt in statements:
            self.newline()
            self.visit(stmt)
        if do_indent:
            self.indentation -= 1

    def body_or_else(self, node):
        self.body(node.body)
        if node.orelse:
            self.newline()
            self.write('else:')
            self.body(node.orelse)

    def signature(self, node):
        want_comma = []

        def write_comma():
            if want_comma:
                self.write(', ')
            else:
                want_comma.append(True)

        padding = [None] * (len(node.args) - len(node.defaults))
        for arg, default in zip(node.args, padding + node.defaults):
            write_comma()
            self.visit(arg)
            if default is not None:
                self.write('=')
                self.visit(default)
        if node.vararg is not None:
            write_comma()
            self.write('*' + node.vararg)
        if node.kwarg is not None:
            write_comma()
            self.write('**' + node.kwarg)

    def decorators(self, node):
        for decorator in node.decorator_list:
            self.newline(decorator)
            self.write('@')
            self.visit(decorator)

    # Statements

    def visit_Assign(self, node):
        self.newline(node)
        for idx, target in enumerate(node.targets):
            if idx:
                self.write(', ')
            self.visit(target)
        self.write(' = ')
        self.visit(node.value)

    def visit_AugAssign(self, node):
        self.newline(node)
        self.visit(node.target)
        self.write(self.BINOP_SYMBOLS[node.op] + '=')
        self.visit(node.value)

    def visit_ImportFrom(self, node):
        self.newline(node)
        self.write('from %s%s import ' % ('.' * node.level, node.module))
        for idx, item in enumerate(node.names):
            if idx:
                self.write(', ')
            self.visit(item)

    def visit_Import(self, node):
        self.newline(node)
        for item in node.names:
            self.write('import ')
            self.visit(item)

    def visit_Expr(self, node):
        self.newline(node)
        self.generic_visit(node)

    def visit_Module(self, node):
        self.body(node.body, do_indent=False)

    def visit_FunctionDef(self, node):
        self.newline(extra=1)
        self.decorators(node)
        self.newline(node)
        self.write('def %s(' % node.name)
        self.signature(node.args)
        self.write('):')
        self.body(node.body)

    def visit_ClassDef(self, node):
        have_args = []

        def paren_or_comma():
            if have_args:
                self.write(', ')
            else:
                have_args.append(True)
                self.write('(')

        self.newline(extra=2)
        self.decorators(node)
        self.newline(node)
        self.write('class %s' % node.name)
        for base in node.bases:
            paren_or_comma()
            self.visit(base)
        # XXX: the if here is used to keep this module compatible
        #      with python 2.6.
        if hasattr(node, 'keywords'):
            for keyword in node.keywords:
                paren_or_comma()
                self.write(keyword.arg + '=')
                self.visit(keyword.value)
            if node.starargs is not None:
                paren_or_comma()
                self.write('*')
                self.visit(node.starargs)
            if node.kwargs is not None:
                paren_or_comma()
                self.write('**')
                self.visit(node.kwargs)
        self.write(have_args and '):' or ':')
        self.body(node.body)

    def visit_If(self, node):
        self.newline(node)
        self.write('if ')
        self.visit(node.test)
        self.write(':')
        self.body(node.body)
        while True:
            else_ = node.orelse
            if len(else_) == 1 and isinstance(else_[0], If):
                node = else_[0]
                self.newline()
                self.write('elif ')
                self.visit(node.test)
                self.write(':')
                self.body(node.body)
            elif else_:
                self.newline()
                self.write('else:')
                self.body(else_)
                break
            else:
                break

    def visit_For(self, node):
        self.newline(node)
        self.write('for ')
        self.visit(node.target)
        self.write(' in ')
        self.visit(node.iter)
        self.write(':')
        self.body_or_else(node)

    def visit_While(self, node):
        self.newline(node)
        self.write('while ')
        self.visit(node.test)
        self.write(':')
        self.body_or_else(node)

    def visit_With(self, node):
        self.newline(node)
        self.write('with ')
        self.visit(node.context_expr)
        if node.optional_vars is not None:
            self.write(' as ')
            self.visit(node.optional_vars)
        self.write(':')
        self.body(node.body)

    def visit_Pass(self, node):
        self.newline(node)
        self.write('pass')

    def visit_Print(self, node):
        # XXX: python 2.6 only
        self.newline(node)
        self.write('print ')
        want_comma = False
        if node.dest is not None:
            self.write(' >> ')
            self.visit(node.dest)
            want_comma = True
        for value in node.values:
            if want_comma:
                self.write(', ')
            self.visit(value)
            want_comma = True
        if not node.nl:
            self.write(',')

    def visit_Delete(self, node):
        self.newline(node)
        self.write('del ')
        for idx, target in enumerate(node):
            if idx:
                self.write(', ')
            self.visit(target)

    def visit_TryExcept(self, node):
        self.newline(node)
        self.write('try:')
        self.body(node.body)
        for handler in node.handlers:
            self.visit(handler)

    def visit_TryFinally(self, node):
        self.newline(node)
        self.write('try:')
        self.body(node.body)
        self.newline(node)
        self.write('finally:')
        self.body(node.finalbody)

    def visit_Global(self, node):
        self.newline(node)
        self.write('global ' + ', '.join(node.names))

    def visit_Nonlocal(self, node):
        self.newline(node)
        self.write('nonlocal ' + ', '.join(node.names))

    def visit_Return(self, node):
        self.newline(node)
        self.write('return')
        if hasattr(node, "value") and node.value is not None:
            self.write(' ')
            self.visit(node.value)

    def visit_Break(self, node):
        self.newline(node)
        self.write('break')

    def visit_Continue(self, node):
        self.newline(node)
        self.write('continue')

    def visit_Raise(self, node):
        # XXX: Python 2.6 / 3.0 compatibility
        self.newline(node)
        self.write('raise')
        if hasattr(node, 'exc') and node.exc is not None:
            self.write(' ')
            self.visit(node.exc)
            if node.cause is not None:
                self.write(' from ')
                self.visit(node.cause)
        elif hasattr(node, 'type') and node.type is not None:
            self.visit(node.type)
            if node.inst is not None:
                self.write(', ')
                self.visit(node.inst)
            if node.tback is not None:
                self.write(', ')
                self.visit(node.tback)

    def visit_Comment(self, node):
        self.newline(node)
        self.write('#' + str(node.text))

    def visit_ExtraCode(self, node):
        self.newline(node)
        self.write(str(node.text))

    # Expressions

    def visit_Attribute(self, node):
        self.visit(node.value)
        self.write('.' + node.attr)

    def visit_Call(self, node):
        want_comma = []

        def write_comma():
            if want_comma:
                self.write(', ')
            else:
                want_comma.append(True)

        self.visit(node.func)
        self.write('(')
        for arg in node.args:
            write_comma()
            self.visit(arg)
        for keyword in node.keywords:
            write_comma()
            self.write(keyword.arg + '=')
            self.visit(keyword.value)
        if node.starargs is not None:
            write_comma()
            self.write('*')
            self.visit(node.starargs)
        if node.kwargs is not None:
            write_comma()
            self.write('**')
            self.visit(node.kwargs)
        self.write(')')
    visit_TypedCall = visit_Call

    def visit_Name(self, node):
        self.write(node.id)
    visit_TypedName = visit_Name

    def visit_Str(self, node):
        self.write(repr(node.s))

    def visit_Bytes(self, node):
        self.write(repr(node.s))

    def visit_Num(self, node):
        self.write(repr(node.n))

    def visit_Tuple(self, node):
        self.write('(')
        idx = -1
        for idx, item in enumerate(node.elts):
            if idx:
                self.write(', ')
            self.visit(item)
        self.write(idx and ')' or ',)')

    def sequence_visit(left, right):
        def visit(self, node):
            self.write(left)
            for idx, item in enumerate(node.elts):
                if idx:
                    self.write(', ')
                self.visit(item)
            self.write(right)
        return visit

    visit_List = sequence_visit('[', ']')
    visit_Set = sequence_visit('{', '}')
    del sequence_visit

    def visit_Dict(self, node):
        self.write('{')
        for idx, (key, value) in enumerate(zip(node.keys, node.values)):
            if idx:
                self.write(', ')
            self.visit(key)
            self.write(': ')
            self.visit(value)
        self.write('}')

    def visit_BinOp(self, node):
        op = self.BINOP_SYMBOLS[node.op]
        # if op in ['+', '-']:
        self.write('(')
        self.visit(node.left)
        self.write(' %s ' % op)
        self.visit(node.right)
        # if op in ['+', '-']:
        self.write(')')

    def visit_BoolOp(self, node):
        self.write('(')
        for idx, value in enumerate(node.values):
            if idx:
                self.write(' %s ' % self.BOOLOP_SYMBOLS[node.op])
            self.visit(value)
        self.write(')')

    def visit_Compare(self, node):
        self.write('(')
        self.visit(node.left)
        for op, right in zip(node.ops, node.comparators):
            self.write(' %s ' % self.CMPOP_SYMBOLS[op])
            self.visit(right)
        self.write(')')

    def visit_UnaryOp(self, node):
        self.write('(')
        op = self.UNARYOP_SYMBOLS[node.op]
        self.write(op)
        if op == 'not':
            self.write(' ')
        self.visit(node.operand)
        self.write(')')

    def visit_Subscript(self, node):
        self.visit(node.value)
        self.write('[')
        self.visit(node.slice)
        self.write(']')
    visit_TypedSubscript = visit_Subscript

    def visit_Index(self, node):
        self.visit(node.value)

    def visit_Slice(self, node):
        if node.lower is not None:
            self.visit(node.lower)
        self.write(':')
        if node.upper is not None:
            self.visit(node.upper)
        if node.step is not None:
            self.write(':')
            if not (isinstance(node.step, Name) and node.step.id == 'None'):
                self.visit(node.step)

    def visit_ExtSlice(self, node):
        for idx, item in node.dims:
            if idx:
                self.write(', ')
            self.visit(item)

    def visit_Yield(self, node):
        self.write('yield ')
        self.visit(node.value)

    def visit_Lambda(self, node):
        self.write('(lambda ')
        self.signature(node.args)
        self.write(': ')
        self.visit(node.body)
        self.write(')')

    def visit_LambdaWithStrBody(self, node):
        self.write('(lambda ')
        for idx, arg in enumerate(node.args):
            if idx:
                self.write(', ')
            self.visit(arg)
        self.write(': ')
        self.write(node.body_str)
        self.write(')')

    def visit_Ellipsis(self, node):
        self.write('Ellipsis')

    def generator_visit(left, right):
        def visit(self, node):
            self.write(left)
            self.visit(node.elt)
            for comprehension in node.generators:
                self.visit(comprehension)
            self.write(right)
        return visit

    visit_ListComp = generator_visit('[', ']')
    visit_GeneratorExp = generator_visit('(', ')')
    visit_SetComp = generator_visit('{', '}')
    del generator_visit

    def visit_DictComp(self, node):
        self.write('{')
        self.visit(node.key)
        self.write(': ')
        self.visit(node.value)
        for comprehension in node.generators:
            self.visit(comprehension)
        self.write('}')

    def visit_IfExp(self, node):
        self.visit(node.body)
        self.write(' if ')
        self.visit(node.test)
        self.write(' else ')
        self.visit(node.orelse)

    def visit_Starred(self, node):
        self.write('*')
        self.visit(node.value)

    def visit_Repr(self, node):
        # XXX: python 2.6 only
        self.write('`')
        self.visit(node.value)
        self.write('`')

    # Helper Nodes

    def visit_alias(self, node):
        self.write(node.name)
        if node.asname is not None:
            self.write(' as ' + node.asname)

    def visit_comprehension(self, node):
        self.write(' for ')
        self.visit(node.target)
        self.write(' in ')
        self.visit(node.iter)
        if node.ifs:
            for if_ in node.ifs:
                self.write(' if ')
                self.visit(if_)

    def visit_excepthandler(self, node):
        self.newline(node)
        self.write('except')
        if node.type is not None:
            self.write(' ')
            self.visit(node.type)
            if node.name is not None:
                self.write(' as ')
                self.visit(node.name)
        self.write(':')
        self.body(node.body)



# Copyright (c) 2011-2013, Walter Bender

# This procedure is invoked when the user-definable block on the
# "extras" palette is selected.

# Usage: Import this code into a Python (user-definable) block; when
# it is run, a u-turn block will be added to the Turtle Palette. You
# can use the u-turn block as you would any other block.


def myblock(tw, args):
    ''' Add a uturn block to the 'turtle' palette '''

    # def_prim takes 3 arguments: the primitive name, the number of
    # arguments -- 0 in this case -- and the function to call -- in this
    # case, we define the _prim_uturn function to set heading += 180.
    def _prim_uturn(tw):
        value = tw.turtles.get_active_turtle().get_heading() + 180
        tw.turtles.get_active_turtle().set_heading(value)
        # We also update the label on the heading block to indicate
        # the current heading value
        if tw.lc.update_values:
            tw.lc.update_label_value('heading', value)

    from TurtleArt.tapalette import make_palette, palette_name_to_index
    from TurtleArt.taprimitive import Primitive, ConstantArg
    from gettext import gettext as _

    # Choose a palette for the new block.
    palette = make_palette('turtle')

    # Create a new block prototype.
    palette.add_block('uturn',
                      style='basic-style-extended-vertical',
                      label=_('uturn'),
                      prim_name='uturn',
                      help_string=_('turns the turtle 180 degrees'))

    # Add its primitive to the LogoCode dictionary.
    tw.lc.def_prim(
        'uturn',
        0,
        Primitive(
            _prim_uturn,
            arg_descs=[
                ConstantArg(tw)]))

    # Regenerate the palette, which will now include the new block.
    tw.show_toolbar_palette(palette_name_to_index('turtle'),
                            regenerate=True)



# Copyright (c) 2013 Marion Zepf

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.


""" Extend the `ast` module to include comments """

import ast


class ExtraCode(ast.stmt):

    """Adds extra content to a primitive needed in Python code, e.g.,
    changes to the turtle (e.g., prim_turtle) require the addition of
    turtle = turtles.get_active_turtle()
    Extends the Python abstract grammar by the following: stmt
    = ExtraContent(string text) | ... """

    _fields = ('text')

    def __init__(self, text="", lineno=1, col_offset=0):
        """ text -- the textual content of the comment, i.e. everything
            directly following the hashtag until the next newline """
        self.text = text
        self.lineno = lineno
        self.col_offset = col_offset


class Comment(ast.stmt):

    """ An inline comment, starting with a hashtag (#).
    Extends the Python abstract grammar by the following:
    stmt = Comment(string text) | ... """

    _fields = ('text')

    def __init__(self, text="", lineno=1, col_offset=0):
        """ text -- the textual content of the comment, i.e. everything
            directly following the hashtag until the next newline """
        self.text = text
        self.lineno = lineno
        self.col_offset = col_offset


class LambdaWithStrBody(ast.Lambda):

    """ Lambda AST whose body is a simple string (not ast.Str).
    Extends the Python abstract grammar by the following:
    expr = LambdaWithStrBody(string body_str, expr* args) | ... """

    def __init__(self, body_str="", args=[], lineno=1, col_offset=0):
        self.body_str = body_str
        self.args = args
        self.lineno = lineno
        self.col_offset = col_offset



#!/usr/bin/env python
# portable serial port access with python
# this is a wrapper module for different platform implementations
#
# (C)2001-2002 Chris Liechti <cliechti@gmx.net>
# this is distributed under a free software license, see license.txt

VERSION = '2.4'

import sys

if sys.platform == 'cli':
    from serialcli import *
else:
    import os
    # chose an implementation, depending on os
    if os.name == 'nt':  # sys.platform == 'win32':
        from serialwin32 import *
    elif os.name == 'posix':
        from .serialposix import *
    elif os.name == 'java':
        from serialjava import *
    else:
        raise Exception(
            "Sorry: no implementation for your platform ('%s') available" %
            os.name)



# Copyright (c) 2010-11, Walter Bender, Tony Forster
#
# This Python block returns with the brightness sensor value in the heap
# a range of parameters can be measured, for example, substitute any of
# these strings for the string in the program below.
#
# /sys/devices/platform/olpc-battery.0/power_supply/olpc-battery/current_now
# /sys/devices/platform/olpc-battery.0/power_supply/olpc-battery/voltage_now
# /sys/devices/platform/dcon/backlight/dcon-bl/actual_brightness


def myblock(tw, x):  # ignores second argument
    import os

    # The light sensor is only available on the XO 1.75
    device = '/sys/devices/platform/olpc-ols.0/level'

    if os.path.exists(device):
        fh = open(device)
        string = fh.read()
        fh.close()
        tw.lc.heap.append(float(string))  # append as a float value to the heap
    else:
        tw.lc.heap.append(-1)

# If you can work out how to use them...
# accelerometer: /dev/input/event0 ???
# power button: /dev/input/event1
# lid switch: /dev/input/event2
# ebook: /dev/input/event3
# headphone jack: /dev/input/event7
# microphone jack: /dev/input/event8
# rotate, cursor, and game pad keys: /dev/input/event10



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Python Serial Port Extension for Win32, Linux, BSD, Jython
# module for serial IO for POSIX compatible systems, like Linux
# see __init__.py
#
# (C) 2001-2008 Chris Liechti <cliechti@gmx.net>
# this is distributed under a free software license, see license.txt
#
# parts based on code from Grant B. Edwards  <grante@visi.com>:
#  ftp://ftp.visi.com/users/grante/python/PosixSerial.py
# references: http://www.easysw.com/~mike/serial/serial.html

import sys
import os
import fcntl
import termios
import struct
import select
import errno
from serialutil import portnum, SerialBase, SerialException, \
    portNotOpenError, writeTimeoutError

from serialutil import VERSION, STOPBITS_ONE, STOPBITS_TWO, \
    PARITY_NONE, PARITY_EVEN, PARITY_ODD, EIGHTBITS

# Do check the Python version as some constants have moved.
if (sys.hexversion < 0x020100f0):
    import TERMIOS
else:
    TERMIOS = termios

if (sys.hexversion < 0x020200f0):
    import FCNTL
else:
    FCNTL = fcntl

# try to detect the os so that a device can be selected...
plat = sys.platform.lower()

if plat[:5] == 'linux':  # Linux (confirmed)
    def device(port):
        return '/dev/ttyS%d' % port

elif plat == 'cygwin':  # cywin/win32 (confirmed)
    def device(port):
        return '/dev/com%d' % (port + 1)

elif plat == 'openbsd3':  # BSD (confirmed)
    def device(port):
        return '/dev/ttyp%d' % port

elif plat[:3] == 'bsd' or  \
        plat[:7] == 'freebsd' or \
        plat[:7] == 'openbsd' or \
        plat[:6] == 'darwin':  # BSD (confirmed for freebsd4: cuaa%d)
    def device(port):
        return '/dev/cuad%d' % port

elif plat[:6] == 'netbsd':  # NetBSD 1.6 testing by Erk
    def device(port):
        return '/dev/dty%02d' % port

elif plat[:4] == 'irix':  # IRIX (partialy tested)
    def device(port):
        # XXX different device names depending on flow control
        return '/dev/ttyf%d' % (port + 1)

elif plat[:2] == 'hp':  # HP-UX (not tested)
    def device(port):
        return '/dev/tty%dp0' % (port + 1)

elif plat[:5] == 'sunos':  # Solaris/SunOS (confirmed)
    def device(port):
        return '/dev/tty%c' % (ord('a') + port)

elif plat[:3] == 'aix':  # aix
    def device(port):
        return '/dev/tty%d' % (port)

else:
    # platform detection has failed...
    print """don't know how to number ttys on this system.
! Use an explicit path (eg /dev/ttyS1) or send this information to
! the author of this module:

sys.platform = %r
os.name = %r
serialposix.py version = %s

also add the device name of the serial port and where the
counting starts for the first serial port.
e.g. 'first serial port: /dev/ttyS0'
and with a bit luck you can get this module running...
""" % (sys.platform, os.name, VERSION)
    # no exception, just continue with a brave attempt to build a device name
    # even if the device name is not correct for the platform it has chances
    # to work using a string with the real device name as port paramter.

    def device(portum):
        return '/dev/ttyS%d' % portnum
    #~ raise Exception, "this module does not run on this platform, sorry."

# whats up with "aix", "beos", ....
# they should work, just need to know the device names.


# load some constants for later use.
# try to use values from TERMIOS, use defaults from linux otherwise
TIOCMGET = hasattr(TERMIOS, 'TIOCMGET') and TERMIOS.TIOCMGET or 0x5415
TIOCMBIS = hasattr(TERMIOS, 'TIOCMBIS') and TERMIOS.TIOCMBIS or 0x5416
TIOCMBIC = hasattr(TERMIOS, 'TIOCMBIC') and TERMIOS.TIOCMBIC or 0x5417
TIOCMSET = hasattr(TERMIOS, 'TIOCMSET') and TERMIOS.TIOCMSET or 0x5418

#TIOCM_LE = hasattr(TERMIOS, 'TIOCM_LE') and TERMIOS.TIOCM_LE or 0x001
TIOCM_DTR = hasattr(TERMIOS, 'TIOCM_DTR') and TERMIOS.TIOCM_DTR or 0x002
TIOCM_RTS = hasattr(TERMIOS, 'TIOCM_RTS') and TERMIOS.TIOCM_RTS or 0x004
#TIOCM_ST = hasattr(TERMIOS, 'TIOCM_ST') and TERMIOS.TIOCM_ST or 0x008
#TIOCM_SR = hasattr(TERMIOS, 'TIOCM_SR') and TERMIOS.TIOCM_SR or 0x010

TIOCM_CTS = hasattr(TERMIOS, 'TIOCM_CTS') and TERMIOS.TIOCM_CTS or 0x020
TIOCM_CAR = hasattr(TERMIOS, 'TIOCM_CAR') and TERMIOS.TIOCM_CAR or 0x040
TIOCM_RNG = hasattr(TERMIOS, 'TIOCM_RNG') and TERMIOS.TIOCM_RNG or 0x080
TIOCM_DSR = hasattr(TERMIOS, 'TIOCM_DSR') and TERMIOS.TIOCM_DSR or 0x100
TIOCM_CD = hasattr(TERMIOS, 'TIOCM_CD') and TERMIOS.TIOCM_CD or TIOCM_CAR
TIOCM_RI = hasattr(TERMIOS, 'TIOCM_RI') and TERMIOS.TIOCM_RI or TIOCM_RNG
#TIOCM_OUT1 = hasattr(TERMIOS, 'TIOCM_OUT1') and TERMIOS.TIOCM_OUT1 or 0x2000
#TIOCM_OUT2 = hasattr(TERMIOS, 'TIOCM_OUT2') and TERMIOS.TIOCM_OUT2 or 0x4000
TIOCINQ = hasattr(TERMIOS, 'FIONREAD') and TERMIOS.FIONREAD or 0x541B

TIOCM_zero_str = struct.pack('I', 0)
TIOCM_RTS_str = struct.pack('I', TIOCM_RTS)
TIOCM_DTR_str = struct.pack('I', TIOCM_DTR)

TIOCSBRK = hasattr(TERMIOS, 'TIOCSBRK') and TERMIOS.TIOCSBRK or 0x5427
TIOCCBRK = hasattr(TERMIOS, 'TIOCCBRK') and TERMIOS.TIOCCBRK or 0x5428

ASYNC_SPD_MASK = 0x1030
ASYNC_SPD_CUST = 0x0030

baudrate_constants = {
    0: 0000000,  # hang up
    50: 0o000001,
    75: 0o000002,
    110: 0o000003,
    134: 0o000004,
    150: 0o000005,
    200: 0o000006,
    300: 0o000007,
    600: 0o000010,
    1200: 0o000011,
    1800: 0o000012,
    2400: 0o000013,
    4800: 0o000014,
    9600: 0o000015,
    19200: 0o000016,
    38400: 0o000017,
    57600: 0o010001,
    115200: 0o010002,
    230400: 0o010003,
    460800: 0o010004,
    500000: 0o010005,
    576000: 0o010006,
    921600: 0o010007,
    1000000: 0o010010,
    1152000: 0o010011,
    1500000: 0o010012,
    2000000: 0o010013,
    2500000: 0o010014,
    3000000: 0o010015,
    3500000: 0o010016,
    4000000: 0o010017
}


class Serial(SerialBase):

    """Serial port class POSIX implementation. Serial port configuration is
    done with termios and fcntl. Runs on Linux and many other Un*x like
    systems."""

    def open(self):
        """Open port with current settings. This may throw a SerialException
           if the port cannot be opened."""
        if self._port is None:
            raise SerialException(
                "Port must be configured before it can be used.")
        self.fd = None
        # open
        try:
            self.fd = os.open(
                self.portstr,
                os.O_RDWR | os.O_NOCTTY | os.O_NONBLOCK)
        except Exception as msg:
            self.fd = None
            raise SerialException(
                "could not open port %s: %s" %
                (self._port, msg))
        # ~ fcntl.fcntl(self.fd, FCNTL.F_SETFL, 0)  #set blocking

        try:
            self._reconfigurePort()
        except BaseException:
            os.close(self.fd)
            self.fd = None
        else:
            self._isOpen = True
        #~ self.flushInput()

    def _reconfigurePort(self):
        """Set communication parameters on opened port."""
        if self.fd is None:
            raise SerialException("Can only operate on a valid port handle")
        custom_baud = None

        vmin = vtime = 0  # timeout is done via select
        if self._interCharTimeout is not None:
            vmin = 1
            vtime = int(self._interCharTimeout * 10)
        try:
            iflag, oflag, cflag, lflag, ispeed, ospeed, cc = termios.tcgetattr(
                self.fd)
        # if a port is nonexistent but has a /dev file, it'll fail here
        except termios.error as msg:
            raise SerialException("Could not configure port: %s" % msg)
        # set up raw mode / no echo / binary
        cflag |= (TERMIOS.CLOCAL | TERMIOS.CREAD)
        lflag &= ~(TERMIOS.ICANON | TERMIOS.ECHO | TERMIOS.ECHOE | TERMIOS.ECHOK |
                   TERMIOS.ECHONL | TERMIOS.ISIG | TERMIOS.IEXTEN)  # |TERMIOS.ECHOPRT
        for flag in ('ECHOCTL', 'ECHOKE'):  # netbsd workaround for Erk
            if hasattr(TERMIOS, flag):
                lflag &= ~getattr(TERMIOS, flag)

        oflag &= ~(TERMIOS.OPOST)
        iflag &= ~(
            TERMIOS.INLCR | TERMIOS.IGNCR | TERMIOS.ICRNL | TERMIOS.IGNBRK)
        if hasattr(TERMIOS, 'IUCLC'):
            iflag &= ~TERMIOS.IUCLC
        if hasattr(TERMIOS, 'PARMRK'):
            iflag &= ~TERMIOS.PARMRK

        # setup baudrate
        try:
            ispeed = ospeed = getattr(TERMIOS, 'B%s' % (self._baudrate))
        except AttributeError:
            try:
                ispeed = ospeed = baudrate_constants[self._baudrate]
            except KeyError:
                #~ raise ValueError('Invalid baud rate: %r' % self._baudrate)
                # may need custom baud rate, it isnt in our list.
                ispeed = ospeed = getattr(TERMIOS, 'B38400')
                custom_baud = int(self._baudrate)  # store for later

        # setup char len
        cflag &= ~TERMIOS.CSIZE
        if self._bytesize == 8:
            cflag |= TERMIOS.CS8
        elif self._bytesize == 7:
            cflag |= TERMIOS.CS7
        elif self._bytesize == 6:
            cflag |= TERMIOS.CS6
        elif self._bytesize == 5:
            cflag |= TERMIOS.CS5
        else:
            raise ValueError('Invalid char len: %r' % self._bytesize)
        # setup stopbits
        if self._stopbits == STOPBITS_ONE:
            cflag &= ~(TERMIOS.CSTOPB)
        elif self._stopbits == STOPBITS_TWO:
            cflag |= (TERMIOS.CSTOPB)
        else:
            raise ValueError(
                'Invalid stopit specification: %r' %
                self._stopbits)
        # setup parity
        iflag &= ~(TERMIOS.INPCK | TERMIOS.ISTRIP)
        if self._parity == PARITY_NONE:
            cflag &= ~(TERMIOS.PARENB | TERMIOS.PARODD)
        elif self._parity == PARITY_EVEN:
            cflag &= ~(TERMIOS.PARODD)
            cflag |= (TERMIOS.PARENB)
        elif self._parity == PARITY_ODD:
            cflag |= (TERMIOS.PARENB | TERMIOS.PARODD)
        else:
            raise ValueError('Invalid parity: %r' % self._parity)
        # setup flow control
        # xonxoff
        if hasattr(TERMIOS, 'IXANY'):
            if self._xonxoff:
                iflag |= (TERMIOS.IXON | TERMIOS.IXOFF)  # |TERMIOS.IXANY)
            else:
                iflag &= ~(TERMIOS.IXON | TERMIOS.IXOFF | TERMIOS.IXANY)
        else:
            if self._xonxoff:
                iflag |= (TERMIOS.IXON | TERMIOS.IXOFF)
            else:
                iflag &= ~(TERMIOS.IXON | TERMIOS.IXOFF)
        # rtscts
        if hasattr(TERMIOS, 'CRTSCTS'):
            if self._rtscts:
                cflag |= (TERMIOS.CRTSCTS)
            else:
                cflag &= ~(TERMIOS.CRTSCTS)
        # try it with alternate constant name
        elif hasattr(TERMIOS, 'CNEW_RTSCTS'):
            if self._rtscts:
                cflag |= (TERMIOS.CNEW_RTSCTS)
            else:
                cflag &= ~(TERMIOS.CNEW_RTSCTS)
        # XXX should there be a warning if setting up rtscts (and xonxoff etc)
        # fails??

        # buffer
        # vmin "minimal number of characters to be read. = for non blocking"
        if vmin < 0 or vmin > 255:
            raise ValueError('Invalid vmin: %r ' % vmin)
        cc[TERMIOS.VMIN] = vmin
        # vtime
        if vtime < 0 or vtime > 255:
            raise ValueError('Invalid vtime: %r' % vtime)
        cc[TERMIOS.VTIME] = vtime
        # activate settings
        termios.tcsetattr(
            self.fd, TERMIOS.TCSANOW, [
                iflag, oflag, cflag, lflag, ispeed, ospeed, cc])

        # apply custom baud rate, if any
        if custom_baud is not None:
            import array
            buf = array.array('i', [0] * 32)

            # get serial_struct
            FCNTL.ioctl(self.fd, TERMIOS.TIOCGSERIAL, buf)

            # set custom divisor
            buf[6] = buf[7] / custom_baud

            # update flags
            buf[4] &= ~ASYNC_SPD_MASK
            buf[4] |= ASYNC_SPD_CUST

            # set serial_struct
            try:
                res = FCNTL.ioctl(self.fd, TERMIOS.TIOCSSERIAL, buf)
            except IOError:
                raise ValueError(
                    'Failed to set custom baud rate: %r' %
                    self._baudrate)

    def close(self):
        """Close port"""
        if self._isOpen:
            if self.fd is not None:
                os.close(self.fd)
                self.fd = None
            self._isOpen = False

    def makeDeviceName(self, port):
        return device(port)

    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -

    def inWaiting(self):
        """Return the number of characters currently in the input buffer."""
        #~ s = fcntl.ioctl(self.fd, TERMIOS.FIONREAD, TIOCM_zero_str)
        s = fcntl.ioctl(self.fd, TIOCINQ, TIOCM_zero_str)
        return struct.unpack('I', s)[0]

    def read(self, size=1):
        """Read size bytes from the serial port. If a timeout is set it may
           return less characters as requested. With no timeout it will block
           until the requested number of bytes is read."""
        if self.fd is None:
            raise portNotOpenError
        read = ''
        if size > 0:
            while len(read) < size:
                # print "\tread(): size",size, "have", len(read)    #debug
                ready, _, _ = select.select([self.fd], [], [], self._timeout)
                if not ready:
                    break  # timeout
                buf = os.read(self.fd, size - len(read))
                read = read + buf
                if (self._timeout >=
                        0 or self._interCharTimeout > 0) and not buf:
                    break  # early abort on timeout
        return read

    def write(self, data):
        """Output the given string over the serial port."""
        if self.fd is None:
            raise portNotOpenError
        if not isinstance(data, str):
            raise TypeError('expected str, got %s' % type(data))
        t = len(data)
        d = data
        while t > 0:
            try:
                if self._writeTimeout is not None and self._writeTimeout > 0:
                    _, ready, _ = select.select(
                        [], [
                            self.fd], [], self._writeTimeout)
                    if not ready:
                        raise writeTimeoutError
                n = os.write(self.fd, d)
                if self._writeTimeout is not None and self._writeTimeout > 0:
                    _, ready, _ = select.select(
                        [], [
                            self.fd], [], self._writeTimeout)
                    if not ready:
                        raise writeTimeoutError
                d = d[n:]
                t = t - n
            except OSError as v:
                if v.errno != errno.EAGAIN:
                    raise

    def flush(self):
        """Flush of file like objects. In this case, wait until all data
           is written."""
        self.drainOutput()

    def flushInput(self):
        """Clear input buffer, discarding all that is in the buffer."""
        if self.fd is None:
            raise portNotOpenError
        termios.tcflush(self.fd, TERMIOS.TCIFLUSH)

    def flushOutput(self):
        """Clear output buffer, aborting the current output and
        discarding all that is in the buffer."""
        if self.fd is None:
            raise portNotOpenError
        termios.tcflush(self.fd, TERMIOS.TCOFLUSH)

    def sendBreak(self, duration=0.25):
        """Send break condition. Timed, returns to idle state after given duration."""
        if self.fd is None:
            raise portNotOpenError
        termios.tcsendbreak(self.fd, int(duration / 0.25))

    def setBreak(self, level=1):
        """Set break: Controls TXD. When active, to transmitting is possible."""
        if self.fd is None:
            raise portNotOpenError
        if level:
            fcntl.ioctl(self.fd, TIOCSBRK)
        else:
            fcntl.ioctl(self.fd, TIOCCBRK)

    def setRTS(self, level=1):
        """Set terminal status line: Request To Send"""
        if self.fd is None:
            raise portNotOpenError
        if level:
            fcntl.ioctl(self.fd, TIOCMBIS, TIOCM_RTS_str)
        else:
            fcntl.ioctl(self.fd, TIOCMBIC, TIOCM_RTS_str)

    def setDTR(self, level=1):
        """Set terminal status line: Data Terminal Ready"""
        if self.fd is None:
            raise portNotOpenError
        if level:
            fcntl.ioctl(self.fd, TIOCMBIS, TIOCM_DTR_str)
        else:
            fcntl.ioctl(self.fd, TIOCMBIC, TIOCM_DTR_str)

    def getCTS(self):
        """Read terminal status line: Clear To Send"""
        if self.fd is None:
            raise portNotOpenError
        s = fcntl.ioctl(self.fd, TIOCMGET, TIOCM_zero_str)
        return struct.unpack('I', s)[0] & TIOCM_CTS != 0

    def getDSR(self):
        """Read terminal status line: Data Set Ready"""
        if self.fd is None:
            raise portNotOpenError
        s = fcntl.ioctl(self.fd, TIOCMGET, TIOCM_zero_str)
        return struct.unpack('I', s)[0] & TIOCM_DSR != 0

    def getRI(self):
        """Read terminal status line: Ring Indicator"""
        if self.fd is None:
            raise portNotOpenError
        s = fcntl.ioctl(self.fd, TIOCMGET, TIOCM_zero_str)
        return struct.unpack('I', s)[0] & TIOCM_RI != 0

    def getCD(self):
        """Read terminal status line: Carrier Detect"""
        if self.fd is None:
            raise portNotOpenError
        s = fcntl.ioctl(self.fd, TIOCMGET, TIOCM_zero_str)
        return struct.unpack('I', s)[0] & TIOCM_CD != 0

    # - - platform specific - - - -

    def drainOutput(self):
        """internal - not portable!"""
        if self.fd is None:
            raise portNotOpenError
        termios.tcdrain(self.fd)

    def nonblocking(self):
        """internal - not portable!"""
        if self.fd is None:
            raise portNotOpenError
        fcntl.fcntl(self.fd, FCNTL.F_SETFL, FCNTL.O_NONBLOCK)

    def fileno(self):
        """For easier of the serial port instance with select.
           WARNING: this function is not portable to different platforms!"""
        if self.fd is None:
            raise portNotOpenError
        return self.fd


if __name__ == '__main__':
    s = Serial(0,
               baudrate=19200,  # baudrate
               bytesize=EIGHTBITS,  # number of databits
               parity=PARITY_EVEN,  # enable parity checking
               stopbits=STOPBITS_ONE,  # number of stopbits
               timeout=3,  # set a timeout value, None for waiting forever
               xonxoff=0,  # enable software flow control
               rtscts=0,  # enable RTS/CTS flow control
               )
    s.setRTS(1)
    s.setDTR(1)
    s.flushInput()
    s.flushOutput()
    s.write('hello')
    print repr(s.read(5))
    print s.inWaiting()
    del s
